"""Provides some details related to the order."""
# :license: MIT, see LICENSE for more details.

import click

from SoftLayer.CLI.account.invoice_detail import get_invoice_table
from SoftLayer.CLI.command import SLCommand as SLCommand
from SoftLayer.CLI import environment
from SoftLayer.CLI import formatting
from SoftLayer.managers import ordering
from SoftLayer import utils


@click.command(cls=SLCommand)
@click.argument('identifier')
@click.option('--details', is_flag=True, default=False, show_default=True,
              help="Shows a very detailed list of charges")
@environment.pass_env
def cli(env, identifier, details):
    """Provides some details related to order owner, date order, cost information, initial invoice."""

    manager = ordering.OrderingManager(env.client)
    order = manager.get_order_detail(identifier)
    order_table = get_order_table(order)

    invoice = order.get('initialInvoice', {})
    top_items = invoice.get('invoiceTopLevelItems', [])
    invoice_id = invoice.get('id')
    invoice_table = get_invoice_table(invoice_id, top_items, details)

    order_table.add_row(['Initial Invoice', invoice_table])

    env.fout(order_table)


def get_order_table(order):
    """Formats a table for billing order"""

    title = f"Order {order.get('id')}"
    date_format = '%Y-%m-%d'
    table = formatting.Table(["Key", "Value"], title=title)
    table.align = 'l'

    ordered_by = "IBM"
    user = order.get('userRecord', None)
    if user:
        ordered_by = f"{user.get('displayName')} ({utils.lookup(user, 'userStatus', 'name')})"
    table.add_row(['Ordered By', ordered_by])

    table.add_row(['Create Date', utils.clean_time(order.get('createDate'), date_format, date_format)])
    table.add_row(['Modify Date', utils.clean_time(order.get('modifyDate'), date_format, date_format)])
    table.add_row(['Order Approval Date', utils.clean_time(order.get('orderApprovalDate'), date_format, date_format)])
    table.add_row(['Status', order.get('status')])
    table.add_row(['Order Total Amount', f"{float(order.get('orderTotalAmount', '0')):.2f}"])
    table.add_row(['Invoice Total Amount',
                   f"{float(order.get('initialInvoice', {}).get('invoiceTotalAmount', '0')):.2f}"])

    items = order.get('items', [])
    item_table = formatting.Table(["Item Description"])
    item_table.align['description'] = 'l'

    for item in items:
        item_table.add_row([item.get('description')])

    table.add_row(['Items', item_table])

    return table



"""Order snapshot space for a block storage volume."""
# :license: MIT, see LICENSE for more details.

import click
import SoftLayer
from SoftLayer.CLI import environment
from SoftLayer.CLI import exceptions


@click.command(cls=SoftLayer.CLI.command.SLCommand, )
@click.argument('volume_id')
@click.option('--iops',
              type=int,
              help='Performance Storage IOPs, between 100 and 6000 in multiples of 100.')
@click.option('--size',
              type=int,
              help='Size of snapshot space to create in GB.',
              required=True)
@click.option('--tier',
              help='Endurance Storage Tier (IOPS per GB) of the block'
              ' volume for which space is ordered [optional, and only'
              ' valid for endurance storage volumes].',
              type=click.Choice(['0.25', '2', '4', '10']))
@click.option('--upgrade',
              type=bool,
              help='Flag to indicate that the order is an upgrade.',
              default=False,
              is_flag=True)
@environment.pass_env
def cli(env, volume_id, size, tier, upgrade, iops):
    """Order snapshot space for a block storage volume."""
    block_manager = SoftLayer.BlockStorageManager(env.client)

    if tier is not None:
        tier = float(tier)

    if iops is not None:
        if iops < 100 or iops > 6000:
            raise exceptions.ArgumentError(f"Invalid value for '--iops' / '-i': '{iops}' is not one "
                                           "of between 100 and 6000.")
        if iops % 100 != 0:
            raise exceptions.ArgumentError(f"Invalid value for '--iops' / '-i': '{iops}' is not a multiple of 100.")

    try:
        order = block_manager.order_snapshot_space(
            volume_id,
            capacity=size,
            tier=tier,
            upgrade=upgrade,
            iops=iops
        )
    except ValueError as ex:
        raise exceptions.ArgumentError(str(ex))

    if 'placedOrder' in order.keys():
        click.echo(f"Order #{order['placedOrder']['id']} placed successfully!")
        for item in order['placedOrder']['items']:
            click.echo(" > %s" % item['description'])
        if 'status' in order['placedOrder'].keys():
            click.echo(" > Order status: %s" % order['placedOrder']['status'])
    else:
        click.echo("Order could not be placed! Please verify your options " +
                   "and try again.")



"""Create a file storage snapshot."""
# :license: MIT, see LICENSE for more details.

import click
import SoftLayer
from SoftLayer.CLI import environment


@click.command(cls=SoftLayer.CLI.command.SLCommand, )
@click.argument('volume_id')
@click.option('--notes', '-n',
              help='Notes to set on the new snapshot')
@environment.pass_env
def cli(env, volume_id, notes):
    """Creates a snapshot on a given volume

    Example::

        slcli file snapshot-create 12345678 --note snapshotforsoftlayer
        This command creates a snapshot for volume with ID 12345678 and with addition note as snapshotforsoftlayer.
    """
    file_storage_manager = SoftLayer.FileStorageManager(env.client)
    snapshot = file_storage_manager.create_snapshot(volume_id, notes=notes)

    if 'id' in snapshot:
        click.echo('New snapshot created with id: %s' % snapshot['id'])
    else:
        click.echo('Error occurred while creating snapshot.\n'
                   'Ensure volume is not failed over or in another '
                   'state which prevents taking snapshots.')



"""Summary and acknowledgement of upcoming and ongoing maintenance events"""
# :license: MIT, see LICENSE for more details.
import click

from SoftLayer.CLI.command import SLCommand as SLCommand
from SoftLayer.CLI import environment
from SoftLayer.CLI import formatting
from SoftLayer.managers.account import AccountManager as AccountManager
from SoftLayer import utils


@click.command(cls=SLCommand)
@click.option('--announcement', is_flag=True, default=False,
              help="Show only announcement events.")
@click.option('--ack-all', is_flag=True, default=False,
              help="Acknowledge every upcoming event. Doing so will turn off the popup in the control portal.")
@click.option('--date-min', help="Earliest date to retrieve events for [MM/DD/YYYY]. Default: 2 days ago.")
@click.option('--planned', is_flag=True, default=False,
              help="Show only planned events.")
@click.option('--unplanned', is_flag=True, default=False,
              help="Show only unplanned events.")
@environment.pass_env
def cli(env, ack_all, planned, unplanned, announcement, date_min):
    """Summary and acknowledgement of upcoming and ongoing maintenance events"""

    if date_min:
        utils.verify_date(date_min)

    manager = AccountManager(env.client)
    planned_events = manager.get_upcoming_events("PLANNED", date_min)
    unplanned_events = manager.get_upcoming_events("UNPLANNED_INCIDENT", date_min)
    announcement_events = manager.get_upcoming_events("ANNOUNCEMENT", date_min)

    add_ack_flag(planned_events, manager, ack_all)
    add_ack_flag(unplanned_events, manager, ack_all)
    add_ack_flag(announcement_events, manager, ack_all)

    if planned:
        env.fout(planned_event_table(planned_events))

    if unplanned:
        env.fout(unplanned_event_table(unplanned_events))

    if announcement:
        env.fout(announcement_event_table(announcement_events))

    if not planned and not unplanned and not announcement:
        env.fout(planned_event_table(planned_events))
        env.fout(unplanned_event_table(unplanned_events))
        env.fout(announcement_event_table(announcement_events))


def add_ack_flag(events, manager, ack_all):
    """Add acknowledgedFlag to the event"""
    if ack_all:
        for event in events:
            result = manager.ack_event(event['id'])
            event['acknowledgedFlag'] = result


def planned_event_table(events):
    """Formats a table for events"""
    planned_table = formatting.Table(['Event Data', 'Id', 'Event ID', 'Subject', 'Status', 'Items', 'Start Date',
                                      'End Date', 'Acknowledged', 'Updates'], title="Planned Events")
    planned_table.align['Subject'] = 'l'
    planned_table.align['Impacted Resources'] = 'l'
    for event in events:
        planned_table.add_row([
            utils.clean_time(event.get('startDate')),
            event.get('id'),
            event.get('systemTicketId'),
            # Some subjects can have \r\n for some reason.
            utils.clean_splitlines(event.get('subject')),
            utils.lookup(event, 'statusCode', 'name'),
            event.get('impactedResourceCount'),
            utils.clean_time(event.get('startDate')),
            utils.clean_time(event.get('endDate')),
            event.get('acknowledgedFlag'),
            event.get('updateCount'),

        ])
    return planned_table


def unplanned_event_table(events):
    """Formats a table for events"""
    unplanned_table = formatting.Table(['Id', 'Event ID', 'Subject', 'Status', 'Items', 'Start Date',
                                        'Last Updated', 'Acknowledged', 'Updates'], title="Unplanned Events")
    unplanned_table.align['Subject'] = 'l'
    unplanned_table.align['Impacted Resources'] = 'l'
    for event in events:
        unplanned_table.add_row([
            event.get('id'),
            event.get('systemTicketId'),
            # Some subjects can have \r\n for some reason.
            utils.clean_splitlines(event.get('subject')),
            utils.lookup(event, 'statusCode', 'name'),
            event.get('impactedResourceCount'),
            utils.clean_time(event.get('startDate')),
            utils.clean_time(event.get('modifyDate')),
            event.get('acknowledgedFlag'),
            event.get('updateCount'),
        ])
    return unplanned_table


def announcement_event_table(events):
    """Formats a table for events"""
    announcement_table = formatting.Table(
        ['Id', 'Event ID', 'Subject', 'Status', 'Items', 'Acknowledged', 'Updates'], title="Announcement Events")
    announcement_table.align['Subject'] = 'l'
    announcement_table.align['Impacted Resources'] = 'l'
    for event in events:
        announcement_table.add_row([
            event.get('id'),
            event.get('systemTicketId'),
            # Some subjects can have \r\n for some reason.
            utils.clean_splitlines(event.get('subject')),
            utils.lookup(event, 'statusCode', 'name'),
            event.get('impactedResourceCount'),
            event.get('acknowledgedFlag'),
            event.get('updateCount')
        ])
    return announcement_table



"""Server order options for a given chassis."""
# :license: MIT, see LICENSE for more details.

import click

from SoftLayer.CLI.command import SLCommand as SLCommand
from SoftLayer.CLI import environment
from SoftLayer.CLI import formatting
from SoftLayer.managers import account
from SoftLayer.managers import hardware
from SoftLayer.managers import network


@click.command(cls=SLCommand)
@click.argument('location', required=False)
@click.option('--prices', '-p', is_flag=True,
              help='Use --prices to list the server item prices, and to list the Item Prices by location,'
                   'add it to the --prices option using location short name, e.g. --prices dal13')
@environment.pass_env
def cli(env, prices, location=None):
    """Server order options for a given chassis."""

    hardware_manager = hardware.HardwareManager(env.client)
    account_manager = account.AccountManager(env.client)
    options = hardware_manager.get_create_options(location)
    routers = account_manager.get_routers(location=location)
    network_manager = network.NetworkManager(env.client)

    pods = network_manager.get_closed_pods()

    tables = []

    # Datacenters
    dc_table = formatting.Table(['Datacenter', 'Value', 'Note'], title="Datacenters")
    dc_table.sortby = 'Value'
    dc_table.align = 'l'
    for location_info in options['locations']:
        closure = []
        for pod in pods:
            if location_info['key'] in str(pod['name']):
                closure.append(pod['name'])

        notes = '-'
        if len(closure) > 0:
            notes = 'closed soon: %s' % (', '.join(closure))
        dc_table.add_row([location_info['name'], location_info['key'], notes])
    tables.append(dc_table)

    tables.append(_preset_prices_table(options['sizes'], prices))
    tables.append(_os_prices_table(options['operating_systems'], prices))
    tables.append(_port_speed_prices_table(options['port_speeds'], prices))
    tables.append(_extras_prices_table(options['extras'], prices))
    tables.append(_get_routers(routers))
    env.fout(tables)


def _preset_prices_table(sizes, prices=False):
    """Shows Server Preset options prices.

    :param [] sizes: List of Hardware Server sizes.
    :param prices: Create a price table or not
    """
    if prices:
        table = formatting.Table(['Size', 'Value', 'Hourly', 'Monthly'], title="Sizes")
        for size in sizes:
            if size.get('hourlyRecurringFee', 0) + size.get('recurringFee', 0) + 1 > 0:
                table.add_row([size['name'], size['key'], "%.4f" % size['hourlyRecurringFee'],
                               "%.4f" % size['recurringFee']])
    else:
        table = formatting.Table(['Size', 'Value'], title="Sizes")
        for size in sizes:
            table.add_row([size['name'], size['key']])
    table.sortby = 'Value'
    table.align = 'l'
    return table


def _os_prices_table(operating_systems, prices=False):
    """Shows Server Operating Systems prices cost and capacity restriction.

    :param [] operating_systems: List of Hardware Server operating systems.
    :param prices: Create a price table or not
    """
    if prices:
        table = formatting.Table(['Key', 'Hourly', 'Monthly', 'Restriction'],
                                 title="Operating Systems")
        for operating_system in operating_systems:
            for price in operating_system['prices']:
                cr_max = _get_price_data(price, 'capacityRestrictionMaximum')
                cr_min = _get_price_data(price, 'capacityRestrictionMinimum')
                cr_type = _get_price_data(price, 'capacityRestrictionType')
                table.add_row(
                    [operating_system['key'],
                     _get_price_data(price, 'hourlyRecurringFee'),
                     _get_price_data(price, 'recurringFee'),
                     "%s - %s %s" % (cr_min, cr_max, cr_type)])
    else:
        table = formatting.Table(['OS', 'Key', 'Reference Code'], title="Operating Systems")
        for operating_system in operating_systems:
            table.add_row([operating_system['name'], operating_system['key'], operating_system['referenceCode']])

    table.sortby = 'Key'
    table.align = 'l'
    return table


def _port_speed_prices_table(port_speeds, prices=False):
    """Shows Server Port Speeds prices cost and capacity restriction.

    :param [] port_speeds: List of Hardware Server Port Speeds.
    :param prices: Create a price table or not
    """
    if prices:
        table = formatting.Table(['Key', 'Speed', 'Hourly', 'Monthly'], title="Network Options")
        for speed in port_speeds:
            for price in speed['prices']:
                table.add_row(
                    [speed['key'], speed['speed'],
                     _get_price_data(price, 'hourlyRecurringFee'),
                     _get_price_data(price, 'recurringFee')])
    else:
        table = formatting.Table(['Network', 'Speed', 'Key'], title="Network Options")
        for speed in port_speeds:
            table.add_row([speed['name'], speed['speed'], speed['key']])
    table.sortby = 'Speed'
    table.align = 'l'
    return table


def _extras_prices_table(extras, prices=False):
    """Shows Server extras prices cost and capacity restriction.

    :param [] extras: List of Hardware Server Extras.
    :param prices: Create a price table or not
    """
    if prices:
        table = formatting.Table(['Key', 'Hourly', 'Monthly'], title="Extras")

        for extra in extras:
            for price in extra['prices']:
                table.add_row(
                    [extra['key'],
                     _get_price_data(price, 'hourlyRecurringFee'),
                     _get_price_data(price, 'recurringFee')])
    else:
        table = formatting.Table(['Extra Option', 'Key'], title="Extras")
        for extra in extras:
            table.add_row([extra['name'], extra['key']])
    table.sortby = 'Key'
    table.align = 'l'
    return table


def _get_price_data(price, item):
    """Get a specific data from HS price.

    :param price: Hardware Server price.
    :param string item: Hardware Server price data.
    """
    result = '-'
    if item in price:
        result = price[item]
    return result


def _get_routers(routers):
    """Get all routers information

    :param routers: Routers data
    """

    table = formatting.Table(["id", "hostname", "name"], title='Routers')
    for router in routers:
        table.add_row([router['id'],
                       router['hostname'],
                       router['topLevelLocation']['longName'], ])
    table.align = 'l'
    return table



"""Cancel an existing object-storage account."""
# :license: MIT, see LICENSE for more details.

import click

import SoftLayer
from SoftLayer.CLI import environment
from SoftLayer.CLI import exceptions
from SoftLayer.CLI import formatting


@click.command(cls=SoftLayer.CLI.command.SLCommand, )
@click.argument('storage-id')
@click.option('--reason', help="An optional reason for cancellation")
@click.option('--immediate',
              is_flag=True,
              help="Cancels the object storage immediately instead "
                   "of on the billing anniversary")
@environment.pass_env
def cli(env, storage_id, reason, immediate):
    """Cancel an existing object storage."""

    block_storage_manager = SoftLayer.BlockStorageManager(env.client)

    if not (env.skip_confirmations or formatting.no_going_back(storage_id)):
        raise exceptions.CLIAbort('Aborted')

    cancelled = block_storage_manager.cancel_block_volume(storage_id,
                                                          reason, immediate)

    if cancelled:
        if immediate:
            click.echo('Object storage with id %s has been marked'
                       ' for immediate cancellation' % storage_id)
        else:
            click.echo('Object storage with id %s has been marked'
                       ' for cancellation' % storage_id)
    else:
        click.echo('Unable to cancel object storage %s' % storage_id)



"""List tickets."""
# :license: MIT, see LICENSE for more details.

import click
from SoftLayer import utils

import SoftLayer
from SoftLayer.CLI import environment
from SoftLayer.CLI import formatting


@click.command(cls=SoftLayer.CLI.command.SLCommand, )
@click.option('--open / --closed', 'is_open', default=True,
              help="Display only open or closed tickets")
@click.option('--limit', default=100, show_default=True, type=click.INT, help="Result limit")
@click.option("--all", "-a", "all_tickets", is_flag=True, default=False, help="Return all tickets")
@environment.pass_env
def cli(env, is_open, limit, all_tickets):
    """List tickets."""
    ticket_mgr = SoftLayer.TicketManager(env.client)
    table = formatting.Table([
        'id', 'Case_Number', 'assigned_user', 'title', 'last_edited', 'status', 'updates', 'priority'
    ])

    tickets = ticket_mgr.list_tickets(open_status=is_open,
                                      closed_status=not is_open, limit=limit, all_tickets=all_tickets)

    for ticket in tickets:
        user = formatting.blank()
        if ticket.get('assignedUser'):
            user = "%s %s" % (ticket['assignedUser']['firstName'], ticket['assignedUser']['lastName'])

        table.add_row([
            ticket['id'],
            ticket['serviceProviderResourceId'],
            user,
            click.wrap_text(ticket['title']),
            utils.clean_time(ticket.get('lastEditDate'), out_format="%Y-%m-%d"),
            ticket['status']['name'],
            ticket.get('updateCount', 0),
            ticket.get('priority', 0)
        ])

    env.fout(table)



"""Delete the credential of an Object Storage Account."""
# :license: MIT, see LICENSE for more details.

import click

import SoftLayer
from SoftLayer.CLI import environment
from SoftLayer.CLI import helpers


@click.command(cls=SoftLayer.CLI.command.SLCommand, )
@click.argument('identifier')
@click.option('--credential_id', '-c', type=click.INT,
              help="This is the credential id associated with the volume")
@environment.pass_env
def cli(env, identifier, credential_id):
    """Delete the credential of an Object Storage Account."""

    mgr = SoftLayer.ObjectStorageManager(env.client)
    storage_id = helpers.resolve_id(mgr.resolve_ids, identifier, 'Object Storage')
    credential = mgr.delete_credential(storage_id, credential_id=credential_id)

    env.fout(credential)



"""Disable scheduled snapshots of a specific volume"""
# :license: MIT, see LICENSE for more details.

import click
import SoftLayer
from SoftLayer.CLI import environment
from SoftLayer.CLI import exceptions


@click.command(cls=SoftLayer.CLI.command.SLCommand, )
@click.argument('volume_id')
@click.option('--schedule-type',
              help='Snapshot schedule [INTERVAL|HOURLY|DAILY|WEEKLY]',
              required=True)
@environment.pass_env
def cli(env, volume_id, schedule_type):
    """Disables snapshots on the specified schedule for a given volume

    Example::

        slcli file snapshot-disable 12345678 -s DAILY
        This command disables daily snapshot for volume with ID 12345678.
    """

    if (schedule_type not in ['INTERVAL', 'HOURLY', 'DAILY', 'WEEKLY']):
        raise exceptions.CLIAbort(
            '--schedule_type must be INTERVAL, HOURLY, DAILY, or WEEKLY')

    file_manager = SoftLayer.FileStorageManager(env.client)
    disabled = file_manager.disable_snapshots(volume_id, schedule_type)

    if disabled:
        click.echo('%s snapshots have been disabled for volume %s'
                   % (schedule_type, volume_id))



getUnattachedTagsForCurrentUser = [{'id': 287895, 'name': 'coreos', 'referenceCount': 0}]
getAttachedTagsForCurrentUser = [{'id': 1286571, 'name': 'bs_test_instance', 'referenceCount': 5}]
getReferences = [
    {
        'id': 73009305,
        'resourceTableId': 33488921,
        'tag': {
            'id': 1286571,
            'name': 'bs_test_instance',
        },
        'tagId': 1286571,
        'tagType': {'description': 'CCI', 'keyName': 'GUEST'},
        'tagTypeId': 2,
        'usrRecordId': 6625205
    }
]

deleteTag = True

setTags = True

getObject = getAttachedTagsForCurrentUser[0]

getTagByTagName = getAttachedTagsForCurrentUser

getAllTagTypes = [
    {
        "description": "Hardware",
        "keyName": "HARDWARE"
    }
]



"""Get subnet details."""
# :license: MIT, see LICENSE for more details.

import click

import SoftLayer
from SoftLayer.CLI import environment
from SoftLayer.CLI import formatting
from SoftLayer.CLI import helpers
from SoftLayer import utils


@click.command(cls=SoftLayer.CLI.command.SLCommand, )
@click.argument('identifier')
@click.option('--no-vs',
              is_flag=True,
              help="Hide virtual server listing")
@click.option('--no-hardware',
              is_flag=True,
              help="Hide hardware listing")
@environment.pass_env
def cli(env, identifier, no_vs, no_hardware):
    """Get subnet details."""

    mgr = SoftLayer.NetworkManager(env.client)
    subnet_id = helpers.resolve_id(mgr.resolve_subnet_ids, identifier,
                                   name='subnet')

    mask = """mask[
networkIdentifier, cidr, subnetType, gateway, broadcastAddress, usableIpAddressCount, note, id,
ipAddresses[
    id, ipAddress, note, isBroadcast, isGateway, isNetwork, isReserved,
    hardware[id, fullyQualifiedDomainName],
    virtualGuest[id, fullyQualifiedDomainName]
],
datacenter[name], networkVlan[networkSpace], tagReferences,
virtualGuests[id, fullyQualifiedDomainName, hostname, domain, primaryIpAddress, primaryBackendIpAddress],
hardware[id, fullyQualifiedDomainName, hostname, domain, primaryIpAddress, primaryBackendIpAddress]
]"""

    subnet = mgr.get_subnet(subnet_id, mask=mask)

    table = formatting.KeyValueTable(['name', 'value'])
    table.align['name'] = 'r'
    table.align['value'] = 'l'

    table.add_row(['id', subnet['id']])
    table.add_row(['identifier', f"{subnet['networkIdentifier']}/{subnet['cidr']}"])
    table.add_row(['subnet type', subnet.get('subnetType', formatting.blank())])
    table.add_row(['network space',  utils.lookup(subnet, 'networkVlan', 'networkSpace')])
    table.add_row(['gateway', subnet.get('gateway', formatting.blank())])
    table.add_row(['broadcast', subnet.get('broadcastAddress', formatting.blank())])
    table.add_row(['datacenter', subnet['datacenter']['name']])
    table.add_row(['usable ips', subnet.get('usableIpAddressCount', formatting.blank())])
    table.add_row(['note', subnet.get('note', formatting.blank())])
    table.add_row(['tags', formatting.tags(subnet.get('tagReferences'))])

    ip_address = subnet.get('ipAddresses')

    ip_table = formatting.KeyValueTable(['id', 'status', 'ip', 'note'])
    for address in ip_address:
        description = '-'
        status = 'Reserved'
        if address.get('isGateway'):
            status = 'Gateway'
        if address.get('isBroadcast'):
            status = 'Broadcast'
        if address.get('isNetwork'):
            status = 'Network'
        if address.get('hardware') is not None:
            description = address['hardware']['fullyQualifiedDomainName']
            status = 'In use'
        elif address.get('virtualGuest') is not None:
            description = address['virtualGuest']['fullyQualifiedDomainName']
            status = 'In use'
        ip_table.add_row([
            address.get('id'), status, f"{address.get('ipAddress')}/{description}", address.get('note', '-')
        ])

    table.add_row(['ipAddresses', ip_table])

    if not no_vs:
        if subnet['virtualGuests']:
            vs_table = formatting.Table(['hostname', 'domain', 'public_ip', 'private_ip'])
            for vsi in subnet['virtualGuests']:
                vs_table.add_row([vsi['hostname'],
                                  vsi['domain'],
                                  vsi.get('primaryIpAddress'),
                                  vsi.get('primaryBackendIpAddress')])
            table.add_row(['vs', vs_table])
        else:
            table.add_row(['vs', formatting.blank()])

    if not no_hardware:
        if subnet['hardware']:
            hw_table = formatting.Table(['hostname', 'domain', 'public_ip', 'private_ip'])
            for hardware in subnet['hardware']:
                hw_table.add_row([hardware['hostname'],
                                  hardware['domain'],
                                  hardware.get('primaryIpAddress'),
                                  hardware.get('primaryBackendIpAddress')])
            table.add_row(['hardware', hw_table])
        else:
            table.add_row(['hardware', formatting.blank()])

    env.fout(table)



"""
    SoftLayer.email
    ~~~~~~~~~~~~~~~~~~~~~~~
    Email manager

    :license: MIT, see License for more details.
"""

from SoftLayer import utils


# Invalid names are ignored due to long method names and short argument names
# pylint: disable=invalid-name


class EmailManager(utils.IdentifierMixin, object):
    """Common functions for getting information from the email service

    :param SoftLayer.API.BaseClient client: the client instance
    """

    def __init__(self, client):
        self.client = client

    def get_account_overview(self, identifier):
        """Gets all the Network Message Delivery Account Overview

        :returns: Network Message Delivery Account overview
        """
        return self.client.call('SoftLayer_Network_Message_Delivery_Email_Sendgrid',
                                'getAccountOverview', id=identifier)

    def get_statistics(self, identifier, days=30):
        """gets statistics from email accounts

        :days: range number
        :returns: statistics Network Message Delivery Account
        """
        body = [["requests", "delivered", "opens", "clicks", "bounds"],
                True,
                True,
                True,
                days
                ]

        return self.client.call('SoftLayer_Network_Message_Delivery_Email_Sendgrid',
                                'getStatistics', id=identifier, *body)

    def get_instance(self, identifier):
        """Gets the Network_Message_Delivery_Email_Sendgrid instance

        :return: Network_Message_Delivery_Email_Sendgrid
        """

        _mask = """emailAddress,type,billingItem,vendor"""

        return self.client.call('SoftLayer_Network_Message_Delivery_Email_Sendgrid',
                                'getObject', id=identifier, mask=_mask)

    def editObject(self, identifier, username=None, password=None):
        """Edit email delivery account related details.

               :param int identifier: The ID of the email account
               :param string username: username of the email account.
               :param string email: email of the email account.
               :param string password: password of the email account to be updated to.
               """
        data = {}
        if username:
            data['username'] = username
        if password:
            data['password'] = password

        return self.client.call('SoftLayer_Network_Message_Delivery_Email_Sendgrid',
                                'editObject', data, id=identifier)

    def update_email(self, identifier, email):
        """Edit email address delivery account .

        :param int identifier: The ID of the email account
        :param string email: email of the email account.

        """
        return self.client.call('SoftLayer_Network_Message_Delivery_Email_Sendgrid',
                                'updateEmailAddress', email, id=identifier)



"""List block storage assigned subnets for the given host id."""
# :license: MIT, see LICENSE for more details.

import click
import SoftLayer
from SoftLayer.CLI import environment
from SoftLayer.CLI import formatting
from SoftLayer.CLI import helpers


COLUMNS = [
    'id',
    'networkIdentifier',
    'cidr'
]


@click.command(cls=SoftLayer.CLI.command.SLCommand, )
@click.argument('access_id', type=int)
@environment.pass_env
def cli(env, access_id):
    """List block storage assigned subnets for the given host id.

    Example::

                slcli block subnets-list 12345678
                ACCESS_ID is the host_id obtained by: softlayer slcli block access-list <volume_id>

    access_id is the host_id obtained by: slcli block access-list <volume_id>
    """

    try:
        block_manager = SoftLayer.BlockStorageManager(env.client)
        resolved_id = helpers.resolve_id(block_manager.resolve_ids, access_id, 'Volume Id')
        subnets = block_manager.get_subnets_in_acl(resolved_id)

        table = formatting.Table(COLUMNS)
        for subnet in subnets:
            row = [f"{subnet['id']}",
                   f"{subnet['networkIdentifier']}",
                   f"{subnet['cidr']}"]
            table.add_row(row)

        env.fout(table)

    except SoftLayer.SoftLayerAPIError as ex:
        message = "Unable to list assigned subnets for access-id: {}.\nReason: {}".format(access_id, ex.faultString)
        click.echo(message)



"""
    SoftLayer.license
    ~~~~~~~~~~~~~~~~~
    License Manager

    :license: MIT, see LICENSE for more details.
"""

# pylint: disable=too-many-public-methods
from SoftLayer.CLI import exceptions
from SoftLayer.managers import ordering
from SoftLayer import utils

LICENSE_PACKAGE_ID = 301


class LicensesManager(object):
    """Manages account license."""

    def __init__(self, client):
        self.client = client

    def get_all_objects(self):
        """Show the all VMware licenses of an account.

         """
        _mask = '''softwareDescription,billingItem'''

        return self.client.call('SoftLayer_Software_AccountLicense',
                                'getAllObjects', mask=_mask)

    def cancel_item(self, key, cancel_immediately=False):
        """Cancel a billing item immediately, deleting all its data.

        :param integer identifier: the instance ID to cancel
        :param string reason_cancel: reason cancel
        """
        vm_ware_licenses = self.get_all_objects()
        vm_ware_find = False
        for vm_ware in vm_ware_licenses:
            if vm_ware.get('key') == key:
                vm_ware_find = True
                self.client.call('SoftLayer_Billing_Item', 'cancelItem',
                                 cancel_immediately,
                                 True,
                                 'Cancel by cli command',
                                 'Cancel by cli command',
                                 id=utils.lookup(vm_ware, 'billingItem', 'id'))

        if not vm_ware_find:
            raise exceptions.CLIAbort(
                f"Unable to find license key: {key}")
        return vm_ware_find

    def create(self, datacenter, item_package):
        """Create a license

        :param string datacenter: the datacenter shortname
        :param string[] item_package: items array
        """
        complex_type = 'SoftLayer_Container_Product_Order_Software_License'
        ordering_manager = ordering.OrderingManager(self.client)
        return ordering_manager.place_order(package_keyname='SOFTWARE_LICENSE_PACKAGE',
                                            location=datacenter,
                                            item_keynames=item_package,
                                            complex_type=complex_type,
                                            hourly=False)

    def get_create_options(self):
        """Returns valid options for ordering Licenses."""
        return self.client.call('SoftLayer_Product_Package', 'getItems', id=LICENSE_PACKAGE_ID)



findByGuestId = [
    {
        'guestId': 100,
        'id': 123,
        'userId': 147852,
        'guest': {
            'domain': 'test.support.com',
            'fullyQualifiedDomainName': 'adns.test.support.com',
            'hostname': 'adns',
            'id': 123,
            'maxCpu': 4,
            'maxCpuUnits': 'CORE',
            'maxMemory': 16384,
            'startCpus': 4,
            'statusId': 1001,
            'uuid': 'b395782d-2144-1f9c-0a90-28a7a5160d81',
        },
        'user': {
            'accountId': 147852,
            'displayName': 'TESTSUPPORT',
            'email': 'testsupport@us.ibm.com',
            'firstName': 'test',
            'id': 167758,
            'lastName': 'support',
            'username': 'test',
        }
    }
]

createObject = {
    'guestId': 100,
    'id': 123,
    'userId': 147852,
    'guest': {
        'domain': 'test.support.com',
        'fullyQualifiedDomainName': 'adns.test.support.com',
        'hostname': 'adns',
        'id': 123,
        'maxCpu': 4,
        'maxCpuUnits': 'CORE',
        'maxMemory': 16384,
        'startCpus': 4,
        'statusId': 1001,
        'uuid': 'b395782d-2144-1f9c-0a90-28a7a5160d81',
    },
    'user': {
        'accountId': 147852,
        'displayName': 'TESTSUPPORT',
        'email': 'testsupport@us.ibm.com',
        'firstName': 'test',
        'id': 167758,
        'lastName': 'support',
        'username': 'test',
    }
}

deleteObjects = True



"""Get monitoring for a firewall device."""
# :license: MIT, see LICENSE for more details.
import datetime

import click

import SoftLayer
from SoftLayer.CLI import environment
from SoftLayer.CLI import formatting


@click.command(cls=SoftLayer.CLI.command.SLCommand, )
@click.argument('identifier')
@environment.pass_env
def cli(env, identifier):
    """Gets bandwidth details for a firewall from the past 30 days.

    Example::
        slcli firewall monitoring vs:12345
    """

    mgr = SoftLayer.FirewallManager(env.client)

    _firewall = mgr.get_instance(identifier)

    table = formatting.KeyValueTable(['name', 'value'])
    table.align['name'] = 'r'
    table.align['value'] = 'l'

    end = datetime.date.today()
    start = end.replace(day=1)
    last_month = start - datetime.timedelta(days=30)

    monitoring = mgr.get_summary(_firewall['metricTrackingObject']['id'],
                                 last_month.strftime('%Y-%m-%d'), end.strftime('%Y-%m-%d'))
    public_out = 0
    public_in = 0
    for monitor in monitoring:

        if monitor['type'] == 'publicOut_net_octet':
            public_out += monitor['counter']

        if monitor['type'] == 'publicIn_net_octet':
            public_in += monitor['counter']

    table.add_row(['Id', _firewall.get('id')])
    table.add_row(['Name', _firewall['networkGateway']['name']])
    table.add_row(['Stard Date', last_month.strftime('%Y-%m-%d')])
    table.add_row(['End Date', end.strftime('%Y-%m-%d')])
    table.add_row(['Out', formatting.b_to_gb(public_out)])
    table.add_row(['In', formatting.b_to_gb(public_in)])
    table.add_row(['Total', formatting.b_to_gb(public_out + public_in)])
    table.add_row(['Status', _firewall['networkGateway']['status']['name']])

    env.fout(table)



"""Reload operating system on a server."""
# :license: MIT, see LICENSE for more details.

import click

import SoftLayer
from SoftLayer.CLI import environment
from SoftLayer.CLI import exceptions
from SoftLayer.CLI import formatting
from SoftLayer.CLI import helpers


@click.command(cls=SoftLayer.CLI.command.SLCommand, )
@click.argument('identifier')
@click.option('--postinstall', '-i',
              help="Post-install script to download (Only HTTPS executes, HTTP leaves file in /root")
@helpers.multi_option('--key', '-k', help="SSH keys to add to the root user")
@click.option('--lvm', '-l', is_flag=True, default=False, show_default=True,
              help="A flag indicating that the provision should use LVM for all logical drives.")
@environment.pass_env
def cli(env, identifier, postinstall, key, lvm):
    """Reload operating system on a server."""

    hardware = SoftLayer.HardwareManager(env.client)
    hardware_id = helpers.resolve_id(hardware.resolve_ids, identifier, 'hardware')
    key_list = []
    if key:
        for single_key in key:
            resolver = SoftLayer.SshKeyManager(env.client).resolve_ids
            key_id = helpers.resolve_id(resolver, single_key, 'SshKey')
            key_list.append(key_id)
    if not (env.skip_confirmations or formatting.no_going_back(hardware_id)):
        raise exceptions.CLIAbort('Aborted')

    hardware.reload(hardware_id, postinstall, key_list, lvm)



"""Order/create a provisioning script."""
# :license: MIT, see LICENSE for more details.

import click

import SoftLayer
from SoftLayer.CLI import environment
from SoftLayer.CLI import formatting


@click.command(cls=SoftLayer.CLI.command.SLCommand)
@click.option('--name', '-N', required=True, prompt=True, help="The name of the hook.")
@click.option('--uri', '-U', required=True, prompt=True, help="The endpoint that the script will be downloaded")
@environment.pass_env
def cli(env, name, uri):
    """Order/create a provisioning script."""

    manager = SoftLayer.AccountManager(env.client)

    provisioning = manager.create_provisioning(name, uri)

    table = formatting.KeyValueTable(['name', 'value'])
    table.align['name'] = 'r'
    table.align['value'] = 'l'

    table.add_row(['Id', provisioning.get('id')])
    table.add_row(['Name', provisioning.get('name')])
    table.add_row(['Created', provisioning.get('createDate')])
    table.add_row(['Uri', provisioning.get('uri')])

    env.fout(table)



getAllObjects = [{
    "capacity": "4",
    "key": "ABCDE-6CJ8L-J8R9H-000R0-CDR70",
    "units": "CPU",
    "billingItem": {
        "allowCancellationFlag": 1,
        "categoryCode": "software_license",
        "createDate": "2018-10-22T11:16:48-06:00",
        "cycleStartDate": "2021-06-03T23:11:22-06:00",
        "description": "vCenter Server Appliance 6.0",
        "id": 123654789,
        "lastBillDate": "2021-06-03T23:11:22-06:00",
        "modifyDate": "2021-06-03T23:11:22-06:00",
        "nextBillDate": "2021-07-03T23:00:00-06:00",
        "orderItemId": 385054741,
        "recurringMonths": 1,
        "serviceProviderId": 1,
    },
    "softwareDescription": {
        "id": 1529,
        "longDescription": "VMware vCenter 6.0",
        "manufacturer": "VMware",
        "name": "vCenter",
        "version": "6.0",
        "requiredUser": "administrator@vsphere.local"
    }
},
    {
        "capacity": "1",
        "key": "CBERT-4RL92-K8999-031K4-AJF5J",
        "units": "CPU",
        "billingItem": {
            "allowCancellationFlag": 1,
            "categoryCode": "software_license",
            "createDate": "2021-06-09T14:51:38-06:00",
            "cycleStartDate": "2021-06-09T14:51:38-06:00",
            "description": "VMware vSAN Advanced Tier III 64 - 124 TB 6.x",
            "id": 369852174,
            "nextBillDate": "2021-07-03T23:00:00-06:00",
            "orderItemId": 836502628,
            "recurringMonths": 1,
            "serviceProviderId": 1,
        },
        "softwareDescription": {
            "id": 1795,
            "longDescription": "VMware Virtual SAN Advanced Tier III 6.2",
            "manufacturer": "VMware",
            "name": "Virtual SAN Advanced Tier III",
            "version": "6.2",
        }
}]



"""List  User Device access."""
import click

import SoftLayer
from SoftLayer.CLI import environment
from SoftLayer.CLI import formatting


@click.command(cls=SoftLayer.CLI.command.SLCommand, )
@click.argument('identifier')
@environment.pass_env
def cli(env, identifier):
    """User Device access."""

    mgr = SoftLayer.UserManager(env.client)
    all_permissions = mgr.get_user_permissions(identifier)

    # verify the table in table
    table = formatting.Table(['Name', 'Value'])
    permission_table = formatting.Table(['KeyName', 'Name'])
    for permission in all_permissions:
        if 'ALL_' in permission['key']:
            permission_table.add_row([permission.get('keyName'), permission.get('name')])

    hardwares = mgr.get_user_hardware(identifier)
    dedicatedhosts = mgr.get_user_dedicated_host(identifier)
    virtual_guests = mgr.get_user_virtuals(identifier)
    hardware_table = formatting.KeyValueTable(['Id', 'Device Name', 'Device type', 'Public Ip', 'Private Ip', 'notes'])
    virtual_table = formatting.KeyValueTable(['Id', 'Device Name', 'Device type', 'Public Ip', 'Private Ip', 'notes'])
    dedicated_table = formatting.KeyValueTable(['Id', 'Device Name', 'Device type', 'notes'])

    hardware_table.align['Device Name'] = 'l'
    dedicated_table.align['Device Name'] = 'l'
    virtual_table.align['Device Name'] = 'l'
    for hardware in hardwares:
        hardware_table.add_row([hardware.get('id'),
                                hardware.get('fullyQualifiedDomainName'),
                                'Bare Metal',
                                hardware.get('primaryIpAddress'),
                                hardware.get('primaryBackendIpAddress'),
                                hardware.get('notes') or '-'])
    for host in dedicatedhosts:
        dedicated_table.add_row([host.get('id'),
                                 host.get('name'),
                                 'Dedicated Host',
                                 host.get('notes') or '-'])
    for virtual in virtual_guests:
        virtual_table.add_row([virtual.get('id'),
                               virtual.get('fullyQualifiedDomainName'),
                               'virtual Guests',
                               virtual.get('primaryIpAddress'),
                               virtual.get('primaryBackendIpAddress'),
                               virtual.get('notes') or '-'])

    table.add_row(['Permission', permission_table])
    table.add_row(['Hardware', hardware_table])
    table.add_row(['Dedicated Host', dedicated_table])
    table.add_row(['Virtual Guest', virtual_table])

    env.fout(table)



"""List hardware servers."""
# :license: MIT, see LICENSE for more details.

import click

import SoftLayer
from SoftLayer.CLI import columns as column_helper
from SoftLayer.CLI import environment
from SoftLayer.CLI import formatting
from SoftLayer.CLI import helpers
from SoftLayer import utils

# pylint: disable=unnecessary-lambda

COLUMNS = [
    column_helper.Column('guid', ('globalIdentifier',)),
    column_helper.Column('primary_ip', ('primaryIpAddress',)),
    column_helper.Column('backend_ip', ('primaryBackendIpAddress',)),
    column_helper.Column('datacenter', ('datacenter', 'name')),
    column_helper.Column(
        'action',
        lambda server: formatting.active_txn(server),
        mask='activeTransaction[id, transactionStatus[name, friendlyName]]'),
    column_helper.Column(
        'owner',
        lambda created_by: utils.lookup(created_by, 'billingItem', 'orderItem', 'order', 'userRecord', 'username'),
        mask='billingItem[id,orderItem[id,order[id,userRecord[username]]]]'),
    column_helper.Column(
        'tags',
        lambda server: formatting.tags(server.get('tagReferences')),
        mask="tagReferences.tag.name"),
]

DEFAULT_COLUMNS = [
    'id',
    'hostname',
    'primary_ip',
    'backend_ip',
    'datacenter',
    'action',
    'owner',
    'tags',
]


@click.command(cls=SoftLayer.CLI.command.SLCommand, )
@click.option('--cpu', '-c', help='Filter by number of CPU cores')
@click.option('--domain', '-D', help='Filter by domain')
@click.option('--datacenter', '-d', help='Filter by datacenter')
@click.option('--hostname', '-H', help='Filter by hostname')
@click.option('--memory', '-m', help='Filter by memory in gigabytes')
@click.option('--network', '-n', help='Filter by network port speed in Mbps')
@click.option('--owner', help='Filter by created_by username')
@click.option('--primary_ip', help='Filter by Primary Ip Address')
@click.option('--backend_ip', help='Filter by Backend Ip Address')
@click.option('--search', is_flag=False, flag_value="", default=None,
              help="Use the more flexible Search API to list instances. See `slcli search --types` for list " +
                   "of searchable fields.")
@helpers.multi_option('--tag', help='Filter by tags')
@click.option('--sortby', help='Column to sort by', default='hostname', show_default=True)
@click.option('--columns',
              callback=column_helper.get_formatter(COLUMNS),
              help='Columns to display. [options: %s]' % ', '.join(column.name for column in COLUMNS),
              default=','.join(DEFAULT_COLUMNS),
              show_default=True)
@click.option('--limit', '-l',
              help='How many results to get in one api call, default is 100',
              default=100,
              show_default=True)
@environment.pass_env
def cli(env, sortby, cpu, domain, datacenter, hostname, memory, network, owner, primary_ip, backend_ip,
        search, tag, columns, limit):
    """List hardware servers."""

    if search is not None:
        object_mask = "mask[resource(SoftLayer_Hardware)]"
        search_manager = SoftLayer.SearchManager(env.client)
        servers = search_manager.search_hadrware_instances(
            hostname=hostname,
            domain=domain,
            datacenter=datacenter,
            tags=tag,
            search_string=search,
            mask=object_mask)

    else:
        manager = SoftLayer.HardwareManager(env.client)
        servers = manager.list_hardware(
            hostname=hostname,
            domain=domain,
            cpus=cpu,
            memory=memory,
            datacenter=datacenter,
            nic_speed=network,
            tags=tag,
            owner=owner,
            public_ip=primary_ip,
            private_ip=backend_ip,
            mask="mask(SoftLayer_Hardware_Server)[%s]" % columns.mask(),
            limit=limit)

    table = formatting.Table(columns.columns)
    table.sortby = sortby
    table.align['created_by'] = 'l'
    table.align['tags'] = 'l'

    for server in servers:
        table.add_row([value or formatting.blank()
                       for value in columns.row(server)])

    env.fout(table)



"""Delete a placement group."""

import click

from SoftLayer.CLI.command import SLCommand as SLCommand
from SoftLayer.CLI import environment
from SoftLayer.CLI import exceptions
from SoftLayer.CLI import formatting
from SoftLayer.CLI import helpers
from SoftLayer.managers.vs import VSManager as VSManager
from SoftLayer.managers.vs_placement import PlacementManager as PlacementManager


@click.command(cls=SLCommand, epilog="Once provisioned, virtual guests can be managed with the slcli vs commands")
@click.argument('identifier')
@click.option('--purge', is_flag=True,
              help="Delete all guests in this placement group. "
                   "The group itself can be deleted once all VMs are fully reclaimed")
@environment.pass_env
def cli(env, identifier, purge):
    """Delete a placement group.

    Placement Group MUST be empty before you can delete it.

    IDENTIFIER can be either the Name or Id of the placement group you want to view
    """
    manager = PlacementManager(env.client)
    group_id = helpers.resolve_id(manager.resolve_ids, identifier, 'placement_group')

    if purge:
        placement_group = manager.get_object(group_id)
        guest_list = ', '.join([guest['fullyQualifiedDomainName'] for guest in placement_group['guests']])
        if len(placement_group['guests']) < 1:
            raise exceptions.CLIAbort(f'No virtual servers were found in placement group {identifier}')

        click.secho(f"You are about to delete the following guests!\n{guest_list}", fg='red')
        if not (env.skip_confirmations or formatting.confirm("This action will cancel all guests! Continue?")):
            raise exceptions.CLIAbort('Aborting virtual server order.')
        vm_manager = VSManager(env.client)
        for guest in placement_group['guests']:
            click.secho(f"Deleting {guest['fullyQualifiedDomainName']}...")
            vm_manager.cancel_instance(guest['id'])
        return

    click.secho(f"You are about to delete the following placement group! {identifier}", fg='red')
    if not (env.skip_confirmations or formatting.confirm("This action will cancel the placement group! Continue?")):
        raise exceptions.CLIAbort('Aborting virtual server order.')
    cancel_result = manager.delete(group_id)
    if cancel_result:
        click.secho(f"Placement Group {identifier} has been cancelled.", fg='green')



getAllObjects = [
    {
        "key": "T_1",
        "keyName": "TICKET_VIEW",
        "name": "View Tickets"
    },
    {
        "key": "T_2",
        "keyName": "TEST",
        "name": "A Testing Permission"
    },
    {
        "key": "T_3",
        "keyName": "TEST_3",
        "name": "A Testing Permission 3"
    },
    {
        "key": "T_4",
        "keyName": "TEST_4",
        "name": "A Testing Permission 4"
    },
    {
        "key": "T_5",
        "keyName": "ACCESS_ALL_HARDWARE",
        "name": "A Testing Permission 5"
    },
    {
        'key': 'ALL_1',
        'keyName': 'ACCESS_ALL_HARDWARE',
        'name': 'All Hardware Access'
    },
    {
        'key': 'A_1',
        'keyName': 'ACCOUNT_SUMMARY_VIEW',
        'name': 'View Account Summary'
    },
    {
        'key': 'A_10',
        'keyName': 'ADD_SERVICE_STORAGE',
        'name': 'Add/Upgrade Storage (StorageLayer)'
    }
]



"""Cancel a license."""
# :licenses: MIT, see LICENSE for more details.

import click
import SoftLayer

from SoftLayer.CLI import environment


@click.command(cls=SoftLayer.CLI.command.SLCommand, )
@click.argument('key')
@click.option('--immediate', is_flag=True, help='Immediate cancellation')
@environment.pass_env
def cli(env, key, immediate):
    """Cancel a license."""

    licenses = SoftLayer.LicensesManager(env.client)

    item = licenses.cancel_item(key, immediate)

    if item:
        env.fout(f"License key: {key} was cancelled.")



"""Details of a specific event, and ability to acknowledge event."""
# :license: MIT, see LICENSE for more details.

import click

from SoftLayer.CLI.command import SLCommand as SLCommand
from SoftLayer.CLI import environment
from SoftLayer.CLI import formatting
from SoftLayer.managers.account import AccountManager as AccountManager
from SoftLayer import utils


@click.command(cls=SLCommand)
@click.argument('identifier')
@click.option('--ack', is_flag=True, default=False,
              help="Acknowledge Event. Doing so will turn off the popup in the control portal")
@environment.pass_env
def cli(env, identifier, ack):
    """Details of a specific event, and ability to acknowledge event."""

    # Print a list of all on going maintenance
    manager = AccountManager(env.client)
    event = manager.get_event(identifier)

    if ack:
        manager.ack_event(identifier)

    env.fout(basic_event_table(event))
    env.fout(impacted_table(event))
    env.fout(update_table(event))


def basic_event_table(event):
    """Formats a basic event table"""
    table = formatting.Table(["Id", "Status", "Type", "Start", "End"],
                             title=utils.clean_splitlines(event.get('subject')))

    table.add_row([
        event.get('id'),
        utils.lookup(event, 'statusCode', 'name'),
        utils.lookup(event, 'notificationOccurrenceEventType', 'keyName'),
        utils.clean_time(event.get('startDate')),
        utils.clean_time(event.get('endDate'))
    ])

    return table


def impacted_table(event):
    """Formats a basic impacted resources table"""
    table = formatting.Table([
        "Type", "Id", "Hostname", "PrivateIp", "Label"
    ])
    for item in event.get('impactedResources', []):
        table.add_row([
            item.get('resourceType'),
            item.get('resourceTableId'),
            item.get('hostname'),
            item.get('privateIp'),
            item.get('filterLabel')
        ])
    return table


def update_table(event):
    """Formats a basic event update table"""
    update_number = 0
    for update in event.get('updates', []):
        update_number = update_number + 1
        header = "======= Update #%s on %s =======" % (update_number, utils.clean_time(update.get('startDate')))
        click.secho(header, fg='green')
        text = update.get('contents')
        # deals with all the \r\n from the API
        click.secho(utils.clean_splitlines(text))



"""Find an IP address and display its subnet and device info."""
# :license: MIT, see LICENSE for more details.

import click

import SoftLayer
from SoftLayer.CLI import environment
from SoftLayer.CLI import exceptions
from SoftLayer.CLI import formatting


@click.command(cls=SoftLayer.CLI.command.SLCommand, )
@click.argument('ip_address')
@environment.pass_env
def cli(env, ip_address):
    """Find an IP address and display its subnet and device info."""

    mgr = SoftLayer.NetworkManager(env.client)

    addr_info = mgr.ip_lookup(ip_address)

    if not addr_info:
        raise exceptions.CLIAbort('Not found')

    table = formatting.KeyValueTable(['name', 'value'])
    table.align['name'] = 'r'
    table.align['value'] = 'l'

    table.add_row(['id', addr_info['id']])
    table.add_row(['ip', addr_info['ipAddress']])

    subnet_table = formatting.KeyValueTable(['name', 'value'])
    subnet_table.align['name'] = 'r'
    subnet_table.align['value'] = 'l'
    subnet_table.add_row(['id', addr_info['subnet']['id']])
    subnet_table.add_row(['identifier',
                          '%s/%s' % (addr_info['subnet']['networkIdentifier'],
                                     str(addr_info['subnet']['cidr']))])
    subnet_table.add_row(['netmask', addr_info['subnet']['netmask']])
    if addr_info['subnet'].get('gateway'):
        subnet_table.add_row(['gateway', addr_info['subnet']['gateway']])
    subnet_table.add_row(['type', addr_info['subnet'].get('subnetType')])

    table.add_row(['subnet', subnet_table])

    if addr_info.get('virtualGuest') or addr_info.get('hardware'):
        device_table = formatting.KeyValueTable(['name', 'value'])
        device_table.align['name'] = 'r'
        device_table.align['value'] = 'l'
        if addr_info.get('virtualGuest'):
            device = addr_info['virtualGuest']
            device_type = 'vs'
        else:
            device = addr_info['hardware']
            device_type = 'server'
        device_table.add_row(['id', device['id']])
        device_table.add_row(['name', device['fullyQualifiedDomainName']])
        device_table.add_row(['type', device_type])
        table.add_row(['device', device_table])
    env.fout(table)



"""List all global IPs."""
# :license: MIT, see LICENSE for more details.

import click

import SoftLayer
from SoftLayer.CLI import environment
from SoftLayer.CLI import formatting


@click.command(cls=SoftLayer.CLI.command.SLCommand, )
@click.option('--ip-version',
              help='Display only IPv4',
              type=click.Choice(['v4', 'v6']))
@environment.pass_env
def cli(env, ip_version):
    """List all global IPs.

    Example::
        slcli globalip list
    """

    mgr = SoftLayer.NetworkManager(env.client)

    table = formatting.Table(['id', 'ip', 'assigned', 'target'])

    version = None
    if ip_version == 'v4':
        version = 4
    elif ip_version == 'v6':
        version = 6

    ips = mgr.list_global_ips(version=version)

    for ip_address in ips:
        assigned = 'No'
        target = 'None'
        if ip_address.get('destinationIpAddress'):
            dest = ip_address['destinationIpAddress']
            assigned = 'Yes'
            target = dest['ipAddress']
            virtual_guest = dest.get('virtualGuest')
            if virtual_guest:
                target += (' (%s)'
                           % virtual_guest['fullyQualifiedDomainName'])
            elif ip_address['destinationIpAddress'].get('hardware'):
                target += (' (%s)'
                           % dest['hardware']['fullyQualifiedDomainName'])

        table.add_row([ip_address['id'],
                       ip_address['ipAddress']['ipAddress'],
                       assigned,
                       target])
    env.fout(table)



"""Remove resource record."""
# :license: MIT, see LICENSE for more details.

import click

import SoftLayer
from SoftLayer.CLI import environment
from SoftLayer.CLI import exceptions
from SoftLayer.CLI import formatting


@click.command(cls=SoftLayer.CLI.command.SLCommand, )
@click.argument('record_id')
@environment.pass_env
def cli(env, record_id):
    """Remove resource record.

    Example::
        slcli dns record-remove 12345678
        This command removes resource record with ID 12345678.
"""

    manager = SoftLayer.DNSManager(env.client)

    if not (env.skip_confirmations or formatting.no_going_back('yes')):
        raise exceptions.CLIAbort("Aborted.")

    manager.delete_record(record_id)



"""Add a new subnet to your account."""
# :license: MIT, see LICENSE for more details.

import click

import SoftLayer
from SoftLayer.CLI import environment
from SoftLayer.CLI import exceptions
from SoftLayer.CLI import formatting


@click.command(cls=SoftLayer.CLI.command.SLCommand, short_help="Add a new subnet to your account")
@click.argument('network', type=click.Choice(['static', 'public', 'private']))
@click.argument('quantity', type=click.INT)
@click.argument('endpoint-id', type=click.INT)
@click.option('--ipv6', '--v6', is_flag=True, help="Order IPv6 Addresses")
@click.option('--test',
              is_flag=True,
              help="Do not order the subnet; just get a quote")
@environment.pass_env
def cli(env, network, quantity, endpoint_id, ipv6, test):
    """Add a new subnet to your account. Valid quantities vary by type.

    \b
    IPv4
    static  - 1, 2, 4, 8, 16, 32, 64, 128, 256
    public  - 4, 8, 16, 32, 64, 128, 256
    private - 4, 8, 16, 32, 64, 128, 256

    \b
    IPv6
    static  - 64
    public  - 64

    \b
    endpoint-id
    static  - Network_Subnet_IpAddress identifier.
    public  - Network_Vlan identifier
    private - Network_Vlan identifier
"""

    mgr = SoftLayer.NetworkManager(env.client)

    if not (test or env.skip_confirmations):
        if not formatting.confirm("This action will incur charges on your "
                                  "account. Continue?"):
            raise exceptions.CLIAbort('Cancelling order.')

    version = 4
    if ipv6:
        version = 6

    try:
        result = mgr.add_subnet(network, quantity=quantity, endpoint_id=endpoint_id, version=version, test_order=test)

    except SoftLayer.SoftLayerAPIError as error:
        raise exceptions.CLIAbort(f"Unable to order {quantity} {network} ipv{version} , error: {error.faultString}")

    table = formatting.Table(['Item', 'cost'])
    table.align['Item'] = 'r'
    table.align['cost'] = 'r'

    total = 0.0
    if 'prices' in result:
        for price in result['prices']:
            total += float(price.get('recurringFee', 0.0))
            rate = "%.2f" % float(price['recurringFee'])

            table.add_row([price['item']['description'], rate])

    table.add_row(['Total monthly cost', "%.2f" % total])
    env.fout(table)



"""Create bandwidth pool."""
# :license: MIT, see LICENSE for more details.
import click

from SoftLayer import BandwidthManager
from SoftLayer.CLI.command import SLCommand as SLCommand
from SoftLayer.CLI import environment
from SoftLayer.CLI import formatting
from SoftLayer import utils

location_groups = {
    "SJC/DAL/WDC/TOR/MON": "US/Canada",
    "AMS/LON/MAD/PAR": "AMS/LON/MAD/PAR",
    "SNG/HKG/OSA/TOK": "SNG/HKG/JPN",
    "SYD": "AUS",
    "MEX": "MEX",
    "SAO": "BRA",
    "CHE": "IND",
    "MIL": "ITA",
    "SEO": "KOR",
    "FRA": "FRA"
}

regions = ['SJC/DAL/WDC/TOR/MON', 'AMS/LON/MAD/PAR', 'SNG/HKG/OSA/TOK', 'SYD', 'MEX', 'SAO', 'CHE', 'MIL', 'SEO', 'FRA']


def check_region_param(ctx, param, value):  # pylint: disable=unused-argument
    """Check if provided region is region group or part of region

    :params string value: Region or Region-Groups
    return string Region-Groups
    """

    region_group = None
    for key in location_groups:
        if value in key or value is key:
            region_group = key

    if region_group:
        return region_group
    else:
        raise click.BadParameter(f"{value} is not a region or part of any region."
                                 " Available Choices: ['SJC/DAL/WDC/TOR/MON', 'AMS/LON/MAD/PAR',"
                                 " 'SNG/HKG/OSA/TOK', 'SYD', 'MEX', 'SAO', 'CHE', 'MIL', 'SEO', 'FRA']")


@click.command(cls=SLCommand)
@click.option('--name', required=True, help="Pool name")
@click.option('--region', required=True,
              help=f"Choose Region/Region-Group {regions}", callback=check_region_param)
@click.help_option('--help', '-h')
@environment.pass_env
def cli(env, name, region):
    """Create bandwidth pool.

    Region can be the full zone name 'SJC/DAL/WDC/TOR/MON', or just a single datacenter like 'SJC'.

    Example::
        slcli bandwidth pool-create --name testPool --region DAL
        slcli bandwidth pool-create --name testPool --region SJC/DAL/WDC/TOR/MON
    """

    manager = BandwidthManager(env.client)
    locations = manager.get_location_group()
    id_location_group = get_id_from_location_group(locations, location_groups[region])
    created_pool = manager.create_pool(name, id_location_group)

    table = formatting.KeyValueTable(['Name', 'Value'])
    table.add_row(['Id', created_pool.get('id')])
    table.add_row(['Name Pool', name])
    table.add_row(['Region', region])
    table.add_row(['Created Date', utils.clean_time(created_pool.get('createDate'))])
    env.fout(table)


def get_id_from_location_group(locations, name):
    """Gets the ID location group, from name"""
    for location in locations:
        if location['name'] == name:
            return location['id']

    return None



"""Account summary."""
# :license: MIT, see LICENSE for more details.

import click

import SoftLayer
from SoftLayer.CLI import environment
from SoftLayer.CLI import formatting


COLUMNS = ['datacenter',
           'hardware',
           'virtual_servers',
           'vlans',
           'subnets',
           'public_ips']


@click.command(cls=SoftLayer.CLI.command.SLCommand, )
@click.option('--sortby',
              help='Column to sort by',
              default='datacenter',
              type=click.Choice(COLUMNS))
@environment.pass_env
def cli(env, sortby):
    """Account summary."""

    mgr = SoftLayer.NetworkManager(env.client)
    datacenters = mgr.summary_by_datacenter()

    table = formatting.Table(COLUMNS)
    table.sortby = sortby

    for name, datacenter in datacenters.items():
        table.add_row([
            name or formatting.blank(),
            datacenter.get('hardware_count', formatting.blank()),
            datacenter.get('virtual_guest_count', formatting.blank()),
            datacenter.get('vlan_count', formatting.blank()),
            datacenter.get('subnet_count', formatting.blank()),
            datacenter.get('public_ip_count', formatting.blank()),
        ])

    env.fout(table)



"""Get storage details for a hardware server."""
# :license: MIT, see LICENSE for more details.

import click

import SoftLayer
from SoftLayer.CLI import environment
from SoftLayer.CLI import formatting
from SoftLayer.CLI import helpers


@click.command(cls=SoftLayer.CLI.command.SLCommand, )
@click.argument('identifier')
@environment.pass_env
def cli(env, identifier):
    """Get storage details for a hardware server."""

    hardware = SoftLayer.HardwareManager(env.client)
    hardware_id = helpers.resolve_id(hardware.resolve_ids, identifier, 'hardware')
    iscsi_storage_data = hardware.get_storage_details(hardware_id, "ISCSI")
    nas_storage_data = hardware.get_storage_details(hardware_id, "NAS")
    storage_credentials = hardware.get_storage_credentials(hardware_id)
    hard_drives = hardware.get_hard_drives(hardware_id)

    table_credentials = formatting.Table(['Username', 'Password', 'IQN'], title="Block Storage Details \n iSCSI")
    if storage_credentials:
        table_credentials.add_row([storage_credentials['credential']['username'],
                                   storage_credentials['credential']['password'],
                                   storage_credentials['name']])

    table_iscsi = formatting.Table(['LUN name', 'capacity', 'Target address', 'Location', 'Notes'])
    for iscsi in iscsi_storage_data:
        table_iscsi.add_row([iscsi['username'], iscsi['capacityGb'],
                             iscsi['serviceResourceBackendIpAddress'],
                             iscsi['allowedHardware'][0]['datacenter']['longName'],
                             iscsi.get('notes', None)])

    table_nas = formatting.Table(['Volume name', 'capacity', 'Host Name', 'Location', 'Notes'],
                                 title="File Storage Details")
    for nas in nas_storage_data:
        table_nas.add_row([nas['username'], nas['capacityGb'],
                           nas['serviceResourceBackendIpAddress'],
                           nas['allowedHardware'][0]['datacenter']['longName'],
                           nas.get('notes', None)])

    table_hard_drives = formatting.Table(['Type', 'Name', 'Capacity', 'Serial #'], title="Other storage details")
    for drives in hard_drives:
        type_drive = drives['hardwareComponentModel']['hardwareGenericComponentModel']['hardwareComponentType']['type']
        name = drives['hardwareComponentModel']['manufacturer'] + " " + drives['hardwareComponentModel']['name']
        capacity = str(drives['hardwareComponentModel']['hardwareGenericComponentModel']['capacity']) + " " + str(
            drives['hardwareComponentModel']['hardwareGenericComponentModel']['units'])
        serial = drives['serialNumber']

        table_hard_drives.add_row([type_drive, name, capacity, serial])

    env.fout(table_credentials)
    env.fout(table_iscsi)
    env.fout(table_nas)
    env.fout(table_hard_drives)



"""Display details for a specified email account."""
# :license: MIT, see LICENSE for more details.

import click

from SoftLayer.CLI.command import SLCommand as SLCommand
# Commented this line until we fix EmailManager.GetStatistics() method in golang plugin
# from SoftLayer.CLI.email.list import build_statistics_table
from SoftLayer.CLI import environment
from SoftLayer.CLI import formatting
from SoftLayer.managers.email import EmailManager
from SoftLayer import utils


@click.command(cls=SLCommand)
@click.argument('identifier')
@environment.pass_env
def cli(env, identifier):
    """Display details for a specified email."""

    email_manager = EmailManager(env.client)
    result = email_manager.get_instance(identifier)

    table = formatting.KeyValueTable(['name', 'value'])
    table.align['name'] = 'r'
    table.align['value'] = 'l'

    table.add_row(['id', result.get('id')])
    table.add_row(['username', result.get('username')])
    table.add_row(['email_address', result.get('emailAddress')])
    table.add_row(['create_date', result.get('createDate')])
    table.add_row(['category_code', utils.lookup(result, 'billingItem', 'categoryCode')])
    table.add_row(['description', utils.lookup(result, 'billingItem', 'description')])
    table.add_row(['type_description', utils.lookup(result, 'type', 'description')])
    table.add_row(['type', utils.lookup(result, 'type', 'keyName')])
    table.add_row(['vendor', utils.lookup(result, 'vendor', 'keyName')])
    # Commented these lines until we fix EmailManager.GetStatistics() method in golang plugin
    # statistics = email_manager.get_statistics(identifier)
    #
    # for statistic in statistics:
    #     table.add_row(['statistics', build_statistics_table(statistic)])
    env.fout(table)



"""Manage LBaaS members."""
import click

import SoftLayer
from SoftLayer.CLI import environment
from SoftLayer.exceptions import SoftLayerAPIError


@click.command(cls=SoftLayer.CLI.command.SLCommand, )
@click.argument('identifier')
@click.option('--member', '-m', required=True, help="Member UUID")
@environment.pass_env
def remove(env, identifier, member):
    """Remove a LBaaS member.

    Member UUID can be found from `slcli lb detail`.
    """

    mgr = SoftLayer.LoadBalancerManager(env.client)

    uuid, _ = mgr.get_lbaas_uuid_id(identifier)
    # Get a member ID to remove

    try:
        mgr.delete_lb_member(uuid, member)
        click.secho(f"Member {member} removed", fg='green')
    except SoftLayerAPIError as exception:
        click.secho(f"ERROR: {exception.faultString}", fg='red')


@click.command(cls=SoftLayer.CLI.command.SLCommand, )
@click.argument('identifier')
@click.option('--private/--public', default=True, required=True, help="Private or public IP of the new member.")
@click.option('--member', '-m', required=True, help="Member IP address.")
@click.option('--weight', '-w', default=50, type=int, help="Weight of this member.")
@environment.pass_env
def add(env, identifier, private, member, weight):
    """Add a new LBaaS members."""

    mgr = SoftLayer.LoadBalancerManager(env.client)
    uuid, _ = mgr.get_lbaas_uuid_id(identifier)
    # Get a server ID to add
    to_add = {"weight": weight}
    if private:
        to_add['privateIpAddress'] = member
    else:
        to_add['publicIpAddress'] = member

    try:
        mgr.add_lb_member(uuid, to_add)
        click.secho(f"Member {member} added", fg='green')
    except SoftLayerAPIError as exception:
        if 'publicIpAddress must be a string' in exception.faultString:
            click.secho("This LB requires a Public IP address for its members and none was supplied", fg='red')
        elif 'privateIpAddress must be a string' in exception.faultString:
            click.secho("This LB requires a Private IP address for its members and none was supplied", fg='red')
        click.secho(f"ERROR: {exception.faultString}", fg='red')



"""Attach devices to a ticket."""
# :license: MIT, see LICENSE for more details.

import click

import SoftLayer
from SoftLayer.CLI import environment
from SoftLayer.CLI import exceptions
from SoftLayer.CLI import helpers


@click.command(cls=SoftLayer.CLI.command.SLCommand, )
@click.argument('identifier', type=int)
@click.option('--hardware', 'hardware_identifier',
              help="The identifier for hardware to attach")
@click.option('--virtual', 'virtual_identifier',
              help="The identifier for a virtual server to attach")
@environment.pass_env
def cli(env, identifier, hardware_identifier, virtual_identifier):
    """Attach devices to a ticket."""
    ticket_mgr = SoftLayer.TicketManager(env.client)

    if hardware_identifier and virtual_identifier:
        raise exceptions.ArgumentError("Cannot attach hardware and a virtual server at the same time")

    if hardware_identifier:
        hardware_mgr = SoftLayer.HardwareManager(env.client)
        hardware_id = helpers.resolve_id(hardware_mgr.resolve_ids, hardware_identifier, 'hardware')
        ticket_mgr.attach_hardware(identifier, hardware_id)
    elif virtual_identifier:
        vs_mgr = SoftLayer.VSManager(env.client)
        vs_id = helpers.resolve_id(vs_mgr.resolve_ids, virtual_identifier, 'VS')
        ticket_mgr.attach_virtual_server(identifier, vs_id)
    else:
        raise exceptions.ArgumentError("Must have a hardware or virtual server identifier to attach")



"""Delete Tags."""
# :license: MIT, see LICENSE for more details.

import click

from SoftLayer.CLI.command import SLCommand as SLCommand
from SoftLayer.CLI import environment
from SoftLayer.managers.tags import TagManager


@click.command(cls=SLCommand)
@click.argument('identifier')
@click.option('--name', required=False, default=False, is_flag=True, show_default=False,
              help='Assume identifier is a tag name. Useful if your tag name is a number.')
@environment.pass_env
def cli(env, identifier, name):
    """Delete a Tag. Tag names that contain spaces need to be encased in quotes"""

    tag_manager = TagManager(env.client)
    tag_name = identifier
    # If the identifier is a int, and user didn't tell us it was a name.
    if str.isdigit(identifier) and not name:
        tag = tag_manager.get_tag(identifier)
        tag_name = tag.get('name', None)

    result = tag_manager.delete_tag(tag_name)
    if result:
        click.secho(f"Tag {tag_name} has been removed", fg='green')
    else:
        click.secho(f"Failed to remove tag {tag_name}", fg='red')



# snapshot_enable.py
"""Create a file storage snapshot [ENABLE]."""

import click
import SoftLayer
from SoftLayer.CLI import environment
from SoftLayer.CLI import exceptions


@click.command(cls=SoftLayer.CLI.command.SLCommand, )
@click.argument('volume_id')
@click.option('--schedule-type',
              help='Snapshot schedule [INTERVAL|HOURLY|DAILY|WEEKLY]',
              required=True)
@click.option('--retention-count',
              help='Number of snapshots to retain',
              required=True)
@click.option('--minute',
              help='Minute of the day when snapshots should be taken',
              default=0)
@click.option('--hour',
              help='Hour of the day when snapshots should be taken',
              default=0)
@click.option('--day-of-week',
              help='Day of the week when snapshots should be taken',
              default='SUNDAY')
@environment.pass_env
def cli(env, volume_id, schedule_type, retention_count,
        minute, hour, day_of_week):
    """Enables snapshots for a given volume on the specified schedule

    Example::

        slcli file snapshot-enable 12345678 -s WEEKLY -c 5 -m 0 --hour 2 -d 0
        This command enables snapshot for volume with ID 12345678, snapshot is
        taken weekly on every Sunday at 2:00, and up to 5 snapshots are retained.
    """
    file_manager = SoftLayer.FileStorageManager(env.client)

    valid_schedule_types = {'INTERVAL', 'HOURLY', 'DAILY', 'WEEKLY'}
    valid_days = {'SUNDAY', 'MONDAY', 'TUESDAY', 'WEDNESDAY', 'THURSDAY',
                  'FRIDAY', 'SATURDAY'}

    if schedule_type not in valid_schedule_types:
        raise exceptions.CLIAbort(
            '--schedule-type must be INTERVAL, HOURLY, ' +
            'DAILY, or WEEKLY, not ' + schedule_type)

    if schedule_type == 'INTERVAL' and (minute < 30 or minute > 59):
        raise exceptions.CLIAbort(
            '--minute value must be between 30 and 59')
    if minute < 0 or minute > 59:
        raise exceptions.CLIAbort(
            '--minute value must be between 0 and 59')
    if hour < 0 or hour > 23:
        raise exceptions.CLIAbort(
            '--hour value must be between 0 and 23')
    if day_of_week not in valid_days:
        raise exceptions.CLIAbort(
            '--day_of_week value must be a valid day (ex: SUNDAY)')

    enabled = file_manager.enable_snapshots(volume_id, schedule_type,
                                            retention_count, minute,
                                            hour, day_of_week)

    if enabled:
        click.echo('%s snapshots have been enabled for volume %s'
                   % (schedule_type, volume_id))



"""Failover an inaccessible file volume to its available replicant volume."""
# :license: MIT, see LICENSE for more details.

import click
import SoftLayer
from SoftLayer.CLI import environment
from SoftLayer.CLI import exceptions
from SoftLayer.CLI import formatting


@click.command(cls=SoftLayer.CLI.command.SLCommand,
               epilog="""Failover an inaccessible file volume to its available replicant volume.
If a volume (with replication) becomes inaccessible due to a disaster event, this method can be used to immediately
failover to an available replica in another location. This method does not allow for failback via API.
After using this method, to failback to the original volume, please open a support ticket.
If you wish to test failover, please use replica-failover.
""")
@click.argument('volume-id')
@click.option('--replicant-id', help="ID of the replicant volume")
@environment.pass_env
def cli(env, volume_id, replicant_id):
    """Failover an inaccessible file volume to its available replicant volume.

    Example::
        slcli file disaster-recovery-failover 12345678 87654321
        This command performs failover operation for volume with ID 12345678 to replica volume with ID 87654321.
"""
    file_storage_manager = SoftLayer.FileStorageManager(env.client)

    click.secho("""WARNING : Failover an inaccessible file volume to its available replicant volume."""
                """If a volume (with replication) becomes inaccessible due to a disaster event,"""
                """this method can be used to immediately failover to an available replica in another location."""
                """This method does not allow for failback via the API."""
                """To failback to the original volume after using this method, open a support ticket."""
                """If you wish to test failover, use replica-failover instead.""", fg='red')

    if not formatting.confirm('Are you sure you want to continue?'):
        raise exceptions.CLIAbort('Aborted.')

    file_storage_manager.disaster_recovery_failover_to_replicant(
        volume_id,
        replicant_id
    )

    click.echo("Disaster Recovery Failover to replicant is now in progress.")



"""
    SoftLayer.sshkey
    ~~~~~~~~~~~~~~~~
    SSH Key Manager/helpers

    :license: MIT, see LICENSE for more details.
"""

from SoftLayer import utils


class SshKeyManager(utils.IdentifierMixin, object):
    """Manages account SSH keys in SoftLayer.

    See product information here:
    https://knowledgelayer.softlayer.com/procedure/ssh-keys

    :param SoftLayer.API.BaseClient client: the client instance
    """

    def __init__(self, client):
        self.client = client
        self.sshkey = client['Security_Ssh_Key']
        self.resolvers = [self._get_ids_from_label]

    def add_key(self, key, label, notes=None):
        """Adds a new SSH key to the account.

        :param string key: The SSH key to add
        :param string label: The label for the key
        :param string notes: Additional notes for the key
        :returns: A dictionary of the new key's information.
        """
        order = {
            'key': key,
            'label': label,
            'notes': notes,
        }

        return self.sshkey.createObject(order)

    def delete_key(self, key_id):
        """Permanently deletes an SSH key from the account.

        :param int key_id: The ID of the key to delete

        """
        return self.sshkey.deleteObject(id=key_id)

    def edit_key(self, key_id, label=None, notes=None):
        """Edits information about an SSH key.

        :param int key_id: The ID of the key to edit
        :param string label: The new label for the key
        :param string notes: Notes to set or change on the key
        :returns: A Boolean indicating success or failure
        """
        data = {}

        if label:
            data['label'] = label

        if notes:
            data['notes'] = notes

        return self.sshkey.editObject(data, id=key_id)

    def get_key(self, key_id):
        """Returns full information about a single SSH key.

        :param int key_id: The ID of the key to retrieve
        :returns: A dictionary of information about the key
        """
        return self.sshkey.getObject(id=key_id)

    def list_keys(self, label=None):
        """Lists all SSH keys on the account.

        :param string label: Filter list based on SSH key label
        :returns: A list of dictionaries with information about each key
        """
        _filter = utils.NestedDict({})
        if label:
            _filter['sshKeys']['label'] = utils.query_filter(label)

        return self.client['Account'].getSshKeys(filter=_filter.to_dict())

    def _get_ids_from_label(self, label):
        """Return sshkey IDs which match the given label."""
        keys = self.list_keys()
        results = []
        for key in keys:
            if key['label'] == label:
                results.append(key['id'])
        return results



"""Usage information of a virtual server."""
# :license: MIT, see LICENSE for more details.

import click

import SoftLayer
from SoftLayer.CLI import environment
from SoftLayer.CLI import exceptions
from SoftLayer.CLI import formatting
from SoftLayer.CLI import helpers
from SoftLayer.utils import clean_time


@click.command(cls=SoftLayer.CLI.command.SLCommand, )
@click.argument('identifier')
@click.option('--start_date', '-s', type=click.STRING, required=True, help="Start Date e.g. 2019-3-4 (yyyy-MM-dd)")
@click.option('--end_date', '-e', type=click.STRING, required=True, help="End Date e.g. 2019-4-2 (yyyy-MM-dd)")
@click.option('--valid_type', '-t', type=click.STRING, required=True,
              help="Metric_Data_Type keyName e.g. CPU0, CPU1, MEMORY_USAGE, etc.")
@click.option('--summary_period', '-p', type=click.INT, default=3600,
              help="300, 600, 1800, 3600, 43200 or 86400 seconds")
@environment.pass_env
def cli(env, identifier, start_date, end_date, valid_type, summary_period):
    """Usage information of a virtual server."""

    vsi = SoftLayer.VSManager(env.client)
    table = formatting.Table(['counter', 'dateTime', 'type'])
    table_average = formatting.Table(['Average'])

    vs_id = helpers.resolve_id(vsi.resolve_ids, identifier, 'VS')

    result = vsi.get_summary_data_usage(vs_id, start_date=start_date, end_date=end_date,
                                        valid_type=valid_type.upper(), summary_period=summary_period)

    if len(result) == 0:
        raise exceptions.CLIAbort('No metric data for this range of dates provided')

    count = 0
    counter = 0.00
    for data in result:
        if valid_type.upper() == "MEMORY_USAGE":
            usage_counter = data['counter'] / 2 ** 30
        else:
            usage_counter = data['counter']

        table.add_row([
            round(usage_counter, 2),
            clean_time(data['dateTime']),
            data['type'],
        ])
        counter = counter + usage_counter
        count = count + 1

    average = counter / count

    env.fout(table_average.add_row([round(average, 2)]))

    env.fout(table_average)
    env.fout(table)



"""
    SoftLayer.dedicatedhost
    ~~~~~~~~~~~~~~~~~~~~~~~
    DH Manager/helpers

    :license: MIT, see License for more details.
"""
import logging

from SoftLayer.exceptions import SoftLayerAPIError
from SoftLayer.exceptions import SoftLayerError
from SoftLayer.managers import ordering
from SoftLayer import utils

# Invalid names are ignored due to long method names and short argument names
# pylint: disable=invalid-name

LOGGER = logging.getLogger(__name__)


class DedicatedHostManager(utils.IdentifierMixin, object):
    """Manages SoftLayer Dedicated Hosts.

        See product information here https://www.ibm.com/cloud/dedicated


    :param SoftLayer.API.BaseClient client: the client instance
    :param SoftLayer.managers.OrderingManager ordering_manager: an optional manager to handle ordering.
                                              If none is provided, one will be auto initialized.
    """

    def __init__(self, client, ordering_manager=None):
        self.client = client
        self.account = client['Account']
        self.host = client['Virtual_DedicatedHost']
        self.guest = client['Virtual_Guest']

        if ordering_manager is None:
            self.ordering_manager = ordering.OrderingManager(client)

    def cancel_host(self, host_id):
        """Cancel a dedicated host immediately, it fails if there are still guests in the host.

        :param host_id: The ID of the dedicated host to be cancelled.
        :return: True on success or an exception

        Example::
            # Cancels dedicated host id 12345
            result = mgr.cancel_host(12345)

        """
        return self.host.deleteObject(id=host_id)

    def cancel_guests(self, host_id):
        """Cancel all guests into the dedicated host immediately.

        To cancel an specified guest use the method VSManager.cancel_instance()

        :param host_id: The ID of the dedicated host.
        :return: The id, fqdn and status of all guests into a dictionary. The status
                 could be 'Cancelled' or an exception message, The dictionary is empty
                 if there isn't any guest in the dedicated host.

        Example::
            # Cancel guests of dedicated host id 12345
            result = mgr.cancel_guests(12345)
        """
        result = []

        guests = self.host.getGuests(id=host_id, mask='id,fullyQualifiedDomainName')

        if guests:
            for vs in guests:
                status_info = {
                    'id': vs['id'],
                    'fqdn': vs['fullyQualifiedDomainName'],
                    'status': self._delete_guest(vs['id'])
                }
                result.append(status_info)

        return result

    def list_guests(self, host_id, tags=None, cpus=None, memory=None, hostname=None,
                    domain=None, local_disk=None, nic_speed=None, public_ip=None,
                    private_ip=None, **kwargs):
        """Retrieve a list of all virtual servers on the dedicated host.

        Example::

            # Print out a list of instances with 4 cpu cores in the host id 12345.

            for vsi in mgr.list_guests(host_id=12345, cpus=4):
               print vsi['fullyQualifiedDomainName'], vsi['primaryIpAddress']

            # Using a custom object-mask. Will get ONLY what is specified
            object_mask = "mask[hostname,monitoringRobot[robotStatus]]"
            for vsi in mgr.list_guests(mask=object_mask,cpus=4):
                print vsi

        :param integer host_id: the identifier of dedicated host
        :param list tags: filter based on list of tags
        :param integer cpus: filter based on number of CPUS
        :param integer memory: filter based on amount of memory
        :param string hostname: filter based on hostname
        :param string domain: filter based on domain
        :param string local_disk: filter based on local_disk
        :param integer nic_speed: filter based on network speed (in MBPS)
        :param string public_ip: filter based on public ip address
        :param string private_ip: filter based on private ip address
        :param dict \\*\\*kwargs: response-level options (mask, limit, etc.)
        :returns: Returns a list of dictionaries representing the matching
                  virtual servers
        """
        if 'mask' not in kwargs:
            items = [
                'id',
                'globalIdentifier',
                'hostname',
                'domain',
                'fullyQualifiedDomainName',
                'primaryBackendIpAddress',
                'primaryIpAddress',
                'lastKnownPowerState.name',
                'hourlyBillingFlag',
                'powerState',
                'maxCpu',
                'maxMemory',
                'datacenter',
                'activeTransaction.transactionStatus[friendlyName,name]',
                'status',
            ]
            kwargs['mask'] = "mask[%s]" % ','.join(items)

        _filter = utils.NestedDict(kwargs.get('filter') or {})

        if tags:
            _filter['guests']['tagReferences']['tag']['name'] = {
                'operation': 'in',
                'options': [{'name': 'data', 'value': tags}],
            }

        if cpus:
            _filter['guests']['maxCpu'] = utils.query_filter(cpus)

        if memory:
            _filter['guests']['maxMemory'] = utils.query_filter(memory)

        if hostname:
            _filter['guests']['hostname'] = utils.query_filter(hostname)

        if domain:
            _filter['guests']['domain'] = utils.query_filter(domain)

        if local_disk is not None:
            _filter['guests']['localDiskFlag'] = (
                utils.query_filter(bool(local_disk)))

        if nic_speed:
            _filter['guests']['networkComponents']['maxSpeed'] = (
                utils.query_filter(nic_speed))

        if public_ip:
            _filter['guests']['primaryIpAddress'] = (
                utils.query_filter(public_ip))

        if private_ip:
            _filter['guests']['primaryBackendIpAddress'] = (
                utils.query_filter(private_ip))

        kwargs['filter'] = _filter.to_dict()
        kwargs['iter'] = True
        return self.host.getGuests(id=host_id, **kwargs)

    def list_instances(self, tags=None, hostname=None,
                       datacenter=None, order=None, owner=None):
        """Retrieve a list of all dedicated hosts on the account

        :param list tags: filter based on list of tags
        :param integer cpus: filter based on number of CPUS
        :param integer memory: filter based on amount of memory
        :param string hostname: filter based on hostname
        :param string disk: filter based on disk
        :param string datacenter: filter based on datacenter
        :param dict \\*\\*kwargs: response-level options (mask, limit, etc.)
        :returns: Returns a list of dictionaries representing the matching dedicated host.

        """
        _filter = utils.NestedDict({})
        if tags:
            _filter['dedicatedHosts']['tagReferences']['tag']['name'] = {
                'operation': 'in',
                'options': [{'name': 'data', 'value': tags}],
            }

        if hostname:
            _filter['dedicatedHosts']['name'] = (
                utils.query_filter(hostname)
            )

        if datacenter:
            _filter['dedicatedHosts']['datacenter']['name'] = (
                utils.query_filter(datacenter))

        if order:
            _filter['dedicatedHosts']['billingItem']['orderItem']['order']['id'] = (
                utils.query_filter(order))

        if owner:
            _filter['dedicatedHosts']['billingItem']['orderItem']['order']['userRecord']['username'] = (
                utils.query_filter(owner))

        object_filter = _filter.to_dict()
        object_mask = "mask[id,name,createDate,cpuCount,diskCapacity,memoryCapacity,guestCount," \
                      "datacenter,backendRouter,allocationStatus,billingItem[orderItem[order[userRecord]]]]"
        return self.account.getDedicatedHosts(mask=object_mask, filter=object_filter)

    def get_host(self, host_id, **kwargs):
        """Get details about a dedicated host.

        :param integer : the host ID
        :returns: A dictionary containing host information.

        Example::

            # Print out host ID 12345.
            dh = mgr.get_host(12345)
            print dh

            # Print out only name and backendRouter for instance 12345
            object_mask = "mask[name,backendRouter[id]]"
            dh = mgr.get_host(12345, mask=mask)
            print dh

        """
        if 'mask' not in kwargs:
            kwargs['mask'] = '''
                id,
                name,
                cpuCount,
                memoryCapacity,
                diskCapacity,
                createDate,
                modifyDate,
                backendRouter[
                    id,
                    hostname,
                    domain
                ],
                billingItem[
                    id,
                    nextInvoiceTotalRecurringAmount,
                    children[
                        categoryCode,
                        nextInvoiceTotalRecurringAmount
                    ],
                    orderItem[
                        id,
                        order.userRecord[
                            username
                        ]
                    ]
                ],
                datacenter[
                    id,
                    name,
                    longName
                ],
                guests[
                    id,
                    hostname,
                    domain,
                    uuid
                ],
                guestCount
            '''

        return self.host.getObject(id=host_id, **kwargs)

    def place_order(self, hostname, domain, location, flavor, hourly, router=None):
        """Places an order for a dedicated host.

        See get_create_options() for valid arguments.

        :param string hostname: server hostname
        :param string domain: server domain name
        :param string location: location (datacenter) name
        :param boolean hourly: True if using hourly pricing (default).
                               False for monthly.
        :param int router: an optional value for selecting a backend router
        """
        create_options = self._generate_create_dict(hostname=hostname,
                                                    router=router,
                                                    domain=domain,
                                                    flavor=flavor,
                                                    datacenter=location,
                                                    hourly=hourly)

        return self.client['Product_Order'].placeOrder(create_options)

    def verify_order(self, hostname, domain, location, hourly, flavor, router=None):
        """Verifies an order for a dedicated host.

        See :func:`place_order` for a list of available options.
        """

        create_options = self._generate_create_dict(hostname=hostname,
                                                    router=router,
                                                    domain=domain,
                                                    flavor=flavor,
                                                    datacenter=location,
                                                    hourly=hourly)

        return self.client['Product_Order'].verifyOrder(create_options)

    def _generate_create_dict(self,
                              hostname=None,
                              domain=None,
                              flavor=None,
                              router=None,
                              datacenter=None,
                              hourly=True):
        """Translates args into a dictionary for creating a dedicated host."""
        package = self._get_package()
        item = self._get_item(package, flavor)

        location = self._get_location(package['regions'], datacenter)
        price = self._get_price(item)

        routers = self._get_backend_router(
            location['location']['locationPackageDetails'], item)

        router = self._get_default_router(routers, router)

        hardware = {
            'hostname': hostname,
            'domain': domain,
            'primaryBackendNetworkComponent': {
                'router': {
                    'id': router
                }
            }
        }

        complex_type = "SoftLayer_Container_Product_Order_Virtual_DedicatedHost"

        order = {
            "complexType": complex_type,
            "quantity": 1,
            'location': location['keyname'],
            'packageId': package['id'],
            'prices': [{'id': price}],
            'hardware': [hardware],
            'useHourlyPricing': hourly,
        }
        return order

    def _get_package(self):
        """Get the package related to simple dedicated host ordering."""
        mask = '''
        items[
            id,
            description,
            prices,
            capacity,
            keyName,
            itemCategory[categoryCode],
            bundleItems[capacity,keyName,categories[categoryCode],hardwareGenericComponentModel[id,
            hardwareComponentType[keyName]]]
        ],
        regions[location[location[priceGroups]]]
        '''

        package_keyname = 'DEDICATED_HOST'
        package = self.ordering_manager.get_package_by_key(package_keyname, mask=mask)

        return package

    def _get_location(self, regions, datacenter):
        """Get the longer key with a short location(datacenter) name."""
        for region in regions:
            # list of locations
            if region['location']['location']['name'] == datacenter:
                return region

        raise SoftLayerError("Could not find valid location for: '%s'" % datacenter)

    def get_create_options(self):
        """Returns valid options for ordering a dedicated host."""
        package = self._get_package()
        # Locations
        locations = []
        for region in package['regions']:
            locations.append({
                'name': region['location']['location']['longName'],
                'key': region['location']['location']['name'],
            })
        # flavors
        dedicated_host = []
        for item in package['items']:
            if item['itemCategory']['categoryCode'] == \
                    'dedicated_virtual_hosts':
                dedicated_host.append({
                    'name': item['description'],
                    'key': item['keyName'],
                })

        return {'locations': locations, 'dedicated_host': dedicated_host}

    def _get_price(self, package):
        """Returns valid price for ordering a dedicated host."""

        for price in package['prices']:
            if not price.get('locationGroupId'):
                return price['id']

        raise SoftLayerError("Could not find valid price")

    def _get_item(self, package, flavor):
        """Returns the item for ordering a dedicated host."""

        for item in package['items']:
            if item['keyName'] == flavor:
                return item

        raise SoftLayerError("Could not find valid item for: '%s'" % flavor)

    def _get_backend_router(self, locations, item):
        """Returns valid router options for ordering a dedicated host."""
        mask = '''
            id,
            hostname
        '''
        cpu_count = item['capacity']
        mem_capacity = {}
        disk_capacity = {}
        gpuComponents = {}
        for capacity in item['bundleItems']:
            for category in capacity['categories']:
                if category['categoryCode'] == 'dedicated_host_ram':
                    mem_capacity = capacity['capacity']
                if category['categoryCode'] == 'dedicated_host_disk':
                    disk_capacity = capacity['capacity']

        for hardwareComponent in item['bundleItems']:
            if hardwareComponent['keyName'].find("GPU") != -1:
                hardwareComponentType = hardwareComponent['hardwareGenericComponentModel']['hardwareComponentType']
                gpuComponents = [
                    {
                        'hardwareComponentModel': {
                            'hardwareGenericComponentModel': {
                                'id': hardwareComponent['hardwareGenericComponentModel']['id'],
                                'hardwareComponentType': {
                                    'keyName': hardwareComponentType['keyName']
                                }
                            }
                        }
                    },
                    {
                        'hardwareComponentModel': {
                            'hardwareGenericComponentModel': {
                                'id': hardwareComponent['hardwareGenericComponentModel']['id'],
                                'hardwareComponentType': {
                                    'keyName': hardwareComponentType['keyName']
                                }
                            }
                        }
                    }
                ]

        if locations is not None:
            for location in locations:
                if location['locationId'] is not None:
                    loc_id = location['locationId']
                    host = {
                        'cpuCount': cpu_count,
                        'memoryCapacity': mem_capacity,
                        'diskCapacity': disk_capacity,
                        'datacenter': {
                            'id': loc_id
                        }
                    }
                    if item['keyName'].find("GPU") != -1:
                        host['pciDevices'] = gpuComponents
                    routers = self.host.getAvailableRouters(host, mask=mask)
                    return routers

        raise SoftLayerError("Could not find available routers")

    def _get_default_router(self, routers, router_name=None):
        """Returns the default router for ordering a dedicated host."""
        if router_name is None:
            for router in routers:
                if router['id'] is not None:
                    return router['id']
        else:
            for router in routers:
                if router['hostname'] == router_name:
                    return router['id']

        raise SoftLayerError("Could not find valid default router")

    def get_router_options(self, datacenter=None, flavor=None):
        """Returns available backend routers for the dedicated host."""
        package = self._get_package()

        location = self._get_location(package['regions'], datacenter)
        item = self._get_item(package, flavor)

        return self._get_backend_router(location['location']['locationPackageDetails'], item)

    def _delete_guest(self, guest_id):
        """Deletes a guest and returns 'Cancelled' or and Exception message"""
        msg = 'Cancelled'
        try:
            self.guest.deleteObject(id=guest_id)
        except SoftLayerAPIError as e:
            msg = 'Exception: ' + e.faultString

        return msg



"""Get details for a ticket."""
# :license: MIT, see LICENSE for more details.

import click

import SoftLayer
from SoftLayer.CLI import environment
from SoftLayer.CLI import helpers
from SoftLayer.CLI import ticket


@click.command(cls=SoftLayer.CLI.command.SLCommand, )
@click.argument('identifier')
@click.option('--count',
              type=click.INT,
              help="Number of updates",
              show_default=True,
              default=10)
@environment.pass_env
def cli(env, identifier, count):
    """Get details for a ticket."""

    is_json = False
    if env.format == 'json':
        is_json = True
    mgr = SoftLayer.TicketManager(env.client)

    ticket_id = helpers.resolve_id(mgr.resolve_ids, identifier, 'ticket')
    env.fout(ticket.get_ticket_results(mgr, ticket_id, is_json, update_count=count))



"""List all zones."""
# :license: MIT, see LICENSE for more details.

import click

import SoftLayer
from SoftLayer.CLI import environment
from SoftLayer.CLI import formatting
from SoftLayer.utils import clean_time


@click.command(cls=SoftLayer.CLI.command.SLCommand, )
@environment.pass_env
def cli(env):
    """List all zones.

    Example::
        slcli dns zone-list
        This command lists all zones under current account.
"""

    manager = SoftLayer.DNSManager(env.client)
    object_mask = "mask[id,name,serial,updateDate,resourceRecordCount]"
    zones = manager.list_zones(mask=object_mask)
    table = formatting.Table(['id', 'zone', 'serial', 'updated', 'records'])
    table.align = 'l'
    for zone in zones:
        zone_serial = str(zone.get('serial'))
        zone_date = zone.get('updateDate', None)
        if zone_date is None:
            # The serial is just YYYYMMDD##, and since there is no createDate, just format it like it was one.
            zone_date = f"{zone_serial[0:4]}-{zone_serial[4:6]}-{zone_serial[6:8]}T00:00:00-06:00"
        table.add_row([
            zone['id'],
            zone['name'],
            zone_serial,
            clean_time(zone_date),
            zone.get('resourceRecordCount', 0)
        ])

    env.fout(table)



"""Delete user."""
# :license: MIT, see LICENSE for more details.

import click

import SoftLayer
from SoftLayer.CLI import environment
from SoftLayer.CLI import helpers


@click.command(cls=SoftLayer.CLI.command.SLCommand, )
@click.argument('identifier')
@environment.pass_env
def cli(env, identifier):
    """Sets a user's status to CANCEL_PENDING, which will immediately disable the account,

    and will eventually be fully removed from the account by an automated internal process.

    Example: slcli user delete userId

    """

    mgr = SoftLayer.UserManager(env.client)

    user_id = helpers.resolve_id(mgr.resolve_ids, identifier, 'username')

    user_template = {'userStatusId': 1021}

    result = mgr.edit_user(user_id, user_template)
    if result:
        click.secho(f"{identifier} deleted successfully", fg='green')
    else:
        click.secho(f"Failed to delete {identifier}", fg='red')



getObject = {
    'primaryRouter': {
        'datacenter': {'id': 1234, 'longName': 'TestDC'},
        'fullyQualifiedDomainName': 'fcr01.TestDC'
    },
    'id': 1234,
    'vlanNumber': 4444,
    'firewallInterfaces': None,
    'billingItem': {
        'allowCancellationFlag': 1,
        'categoryCode': 'network_vlan',
        'description': 'Private Network Vlan',
        'id': 235689,
        'notes': 'test cli',
        'orderItemId': 147258,
    },
    'subnets': [
        {
            'broadcastAddress': '169.46.22.127',
            'cidr': 28,
            'gateway': '169.46.22.113',
            'id': 1804813,
            'netmask': '255.255.255.240',
            'networkIdentifier': '169.46.22.112',
            'networkVlanId': 1404267,
            'subnetType': 'ADDITIONAL_PRIMARY',
            'totalIpAddresses': '16',
            'usableIpAddressCount': '13',
            'addressSpace': 'PUBLIC'
        },
        {
            'broadcastAddress': '150.239.7.191',
            'cidr': 27,
            'gateway': '150.239.7.161',
            'id': 2415982,
            'netmask': '255.255.255.224',
            'networkIdentifier': '150.239.7.160',
            'networkVlanId': 1404267,
            'subnetType': 'SECONDARY_ON_VLAN',
            'totalIpAddresses': '32',
            'usableIpAddressCount': '29',
            'version': 4,
            'addressSpace': 'PUBLIC'
        }],
    'hardware': [],
    'virtualGuests': [],
    'tagReferences': [],
}

getObject_no_hard_no_vs_no_trunk = {'primaryRouter': {'fullyQualifiedDomainName': 'bcr02a.fra02.softlayer.com',
                                                      'id': 1186647,
                                                      'datacenter': {'id': 449506,
                                                                     'longName': 'Frankfurt 2',
                                                                     'name': 'fra02'}
                                                      },
                                    'id': 1186647,
                                    'vlanNumber': 932,
                                    'networkVlanFirewall': {
                                        'datacenter': {'id': 449506, 'longName': 'Frankfurt 2'},
                                        'fullyQualifiedDomainName': 'bcr02a.fra02.softlayer.com'
                                        },
                                    'subnets': [
                                        {'gateway': '10.85.154.1',
                                         'id': 1688651,
                                         'netmask': '255.255.255.192',
                                         'networkIdentifier': '10.85.154.0',
                                         'subnetType': 'ADDITIONAL_PRIMARY',
                                         'usableIpAddressCount': '61'}],
                                    'hardware': [
                                        {'domain': 'vmware.chechu.com',
                                         'hostname': 'host15',
                                         'primaryBackendIpAddress': '10.177.122.170',
                                         'primaryIpAddress': '169.46.48.107'}],
                                    'virtualGuests': [
                                        {'domain': 'ocp-virt.chechu.me',
                                         'hostname': 'haproxy',
                                         'primaryBackendIpAddress': '10.85.154.44',
                                         'primaryIpAddress': '158.177.72.67'}],
                                    'networkComponentTrunks': [
                                        {'networkComponent':
                                            {'hardwareId': 1483895, 'networkVlanId': 3285524, 'port': 12,
                                             'downlinkComponent': {'networkVlanId': '', 'port': 0,
                                                                   'primaryIpAddress': '10.194.250.148',
                                                                   'hardware':
                                                                       {
                                                                           'domain': 'chechu.me',
                                                                           'hostname': 'fra02-ocp-virt',
                                                                           'tagReferences': []
                                                                           },
                                                                       'networkComponentGroup': {
                                                                           'groupTypeId': 1,
                                                                           'membersDescription': 'eth0/2'}}}}],
                                    'tagReferences': []}

editObject = True
setTags = True
getList = [getObject]

cancel = True

getCancelFailureReasons = [
    "1 bare metal server(s) still on the VLAN ",
    "1 virtual guest(s) still on the VLAN "
]



"""Order a duplicate block storage volume."""
# :license: MIT, see LICENSE for more details.

import click
import SoftLayer
from SoftLayer.CLI import environment
from SoftLayer.CLI import exceptions


CONTEXT_SETTINGS = {'token_normalize_func': lambda x: x.upper()}


@click.command(cls=SoftLayer.CLI.command.SLCommand, context_settings=CONTEXT_SETTINGS)
@click.argument('origin-volume-id')
@click.option('--origin-snapshot-id', '-o',
              type=int,
              help="ID of an origin volume snapshot to use for duplcation.")
@click.option('--duplicate-size', '-c',
              type=int,
              help='Size of duplicate block volume in GB. '
                   '***If no size is specified, the size of '
                   'the origin volume will be used.***\n'
                   'Potential Sizes: [20, 40, 80, 100, 250, '
                   '500, 1000, 2000, 4000, 8000, 12000] '
                   'Minimum: [the size of the origin volume]')
@click.option('--duplicate-iops', '-i',
              type=int,
              help='Performance Storage IOPS, between 100 and 6000 in '
                   'multiples of 100 [only used for performance volumes] '
                   '***If no IOPS value is specified, the IOPS value of the '
                   'origin volume will be used.***\n'
                   'Requirements: [If IOPS/GB for the origin volume is less '
                   'than 0.3, IOPS/GB for the duplicate must also be less '
                   'than 0.3. If IOPS/GB for the origin volume is greater '
                   'than or equal to 0.3, IOPS/GB for the duplicate must '
                   'also be greater than or equal to 0.3.]')
@click.option('--duplicate-tier', '-t',
              help='Endurance Storage Tier (IOPS per GB) [only used for '
                   'endurance volumes] ***If no tier is specified, the tier '
                   'of the origin volume will be used.***\n'
                   'Requirements: [If IOPS/GB for the origin volume is 0.25, '
                   'IOPS/GB for the duplicate must also be 0.25. If IOPS/GB '
                   'for the origin volume is greater than 0.25, IOPS/GB '
                   'for the duplicate must also be greater than 0.25.]',
              type=click.Choice(['0.25', '2', '4', '10']))
@click.option('--duplicate-snapshot-size', '-s',
              type=int,
              help='The size of snapshot space to order for the duplicate. '
                   '***If no snapshot space size is specified, the snapshot '
                   'space size of the origin block volume will be used.***\n'
                   'Input "0" for this parameter to order a duplicate volume '
                   'with no snapshot space.')
@click.option('--billing',
              type=click.Choice(['hourly', 'monthly']),
              default='monthly',
              help="Optional parameter for Billing rate (default to monthly)")
@click.option('--dependent-duplicate',
              type=click.BOOL,
              default=False,
              show_default=True,
              help='Whether or not this duplicate will be a dependent duplicate '
                   'of the origin volume.')
@environment.pass_env
def cli(env, origin_volume_id, origin_snapshot_id, duplicate_size,
        duplicate_iops, duplicate_tier, duplicate_snapshot_size, billing,
        dependent_duplicate):
    """Order a duplicate block storage volume.

    Example::
        slcli block volume-duplicate 12345678
        This command shows order a new volume by duplicating the volume with ID 12345678.
"""

    block_manager = SoftLayer.BlockStorageManager(env.client)

    hourly_billing_flag = False
    if billing.lower() == "hourly":
        hourly_billing_flag = True

    if duplicate_tier is not None:
        duplicate_tier = float(duplicate_tier)

    try:
        order = block_manager.order_duplicate_volume(
            origin_volume_id,
            origin_snapshot_id=origin_snapshot_id,
            duplicate_size=duplicate_size,
            duplicate_iops=duplicate_iops,
            duplicate_tier_level=duplicate_tier,
            duplicate_snapshot_size=duplicate_snapshot_size,
            hourly_billing_flag=hourly_billing_flag,
            dependent_duplicate=dependent_duplicate
        )
    except ValueError as ex:
        raise exceptions.ArgumentError(str(ex))

    if 'placedOrder' in order.keys():
        click.echo(f"Order #{order['placedOrder']['id']} placed successfully!")
        for item in order['placedOrder']['items']:
            click.echo(" > %s" % item['description'])
    else:
        click.echo("Order could not be placed! Please verify your options " +
                   "and try again.")



"""List all cancelation."""
# :license: MIT, see LICENSE for more details.
import click

from SoftLayer.CLI.command import SLCommand as SLCommand
from SoftLayer.CLI import environment
from SoftLayer.CLI import formatting
from SoftLayer.managers import ordering
from SoftLayer import utils


@click.command(cls=SLCommand)
@environment.pass_env
def cli(env):
    """List all account cancelations"""
    table = formatting.Table([
        'Case Number', 'Items', 'Created', 'Status', 'Requested by'])

    manager = ordering.OrderingManager(env.client)
    cancelations = manager.get_all_cancelation()

    for item in cancelations:
        table.add_row([item.get('ticketId'), item.get('itemCount'),
                       utils.clean_time(item.get('createDate'),
                                        in_format='%Y-%m-%dT%H:%M:%S', out_format='%Y-%m-%d %H:%M'),
                       item['status']['name'],
                       item['user']['firstName'] + ' ' + item['user']['lastName']
                       ])

    env.fout(table)



deleteObject = True
editObject = True
getObject = {'id': 1234,
             'fingerprint': 'aa:bb:cc:dd',
             'label': 'label',
             'notes': 'notes',
             'key': 'ssh-rsa AAAAB3N...pa67 user@example.com'}
createObject = getObject
getAllObjects = [getObject]



"""Add a new SSH key."""
# :license: MIT, see LICENSE for more details.
from os import path

import click

import SoftLayer
from SoftLayer.CLI import environment
from SoftLayer.CLI import exceptions


@click.command(cls=SoftLayer.CLI.command.SLCommand, )
@click.argument('label')
@click.option('--in-file', '-f',
              type=click.Path(exists=True),
              help="The id_rsa.pub file to import for this key")
@click.option('--key', '-k', help="The actual SSH key")
@click.option('--note', help="Extra note that will be associated with key")
@environment.pass_env
def cli(env, label, in_file, key, note):
    """Add a new SSH key."""

    if in_file is None and key is None:
        raise exceptions.ArgumentError(
            'Either [-f | --in-file] or [-k | --key] arguments are required to add a key'
        )

    if in_file and key:
        raise exceptions.ArgumentError(
            '[-f | --in-file] is not allowed with [-k | --key]'
        )

    if key:
        key_text = key
    else:
        with open(path.expanduser(in_file), encoding="utf-8") as key_file:
            key_text = key_file.read().strip()
            key_file.close()

    mgr = SoftLayer.SshKeyManager(env.client)
    result = mgr.add_key(key_text, label, note)

    env.fout("SSH key added: %s" % result.get('fingerprint'))



"""Display a list of cancellation reasons."""
# :license: MIT, see LICENSE for more details.

import click

import SoftLayer
from SoftLayer.CLI import environment
from SoftLayer.CLI import formatting


@click.command(cls=SoftLayer.CLI.command.SLCommand, )
@environment.pass_env
def cli(env):
    """Display a list of cancellation reasons."""

    table = formatting.Table(['Code', 'Reason'])
    table.align['Code'] = 'r'
    table.align['Reason'] = 'l'

    mgr = SoftLayer.HardwareManager(env.client)

    for code, reason in mgr.get_cancellation_reasons().items():
        table.add_row([code, reason])

    env.fout(table)



"""Allows to create, remove or refresh user's API authentication key."""
import click

import SoftLayer
from SoftLayer.CLI import environment
from SoftLayer.CLI import exceptions
from SoftLayer.CLI import helpers


@click.command(cls=SoftLayer.CLI.command.SLCommand, )
@click.argument('identifier')
@click.option('--add', is_flag=True, default=False,
              help="Create an user's API authentication key.")
@click.option('--remove', is_flag=True, default=False,
              help="Remove an user's API authentication key.")
@click.option('--refresh', is_flag=True, default=False,
              help="Refresh an user's API authentication key.")
@environment.pass_env
def cli(env, identifier, add, remove, refresh):

    """Allows to create, remove or refresh user's API authentication key.

    Each user can only have a single API key.
    """

    options = {
        '--add': add,
        '--remove': remove,
        '--refresh': refresh,
    }

    if not any(options.values()):
        raise exceptions.CLIAbort(f'At least one option is required: [{",".join(options.keys())}]')

    if sum(options.values()) > 1:
        raise exceptions.CLIAbort(f'Can only specify one option of [{",".join(options.keys())}]')

    mgr = SoftLayer.UserManager(env.client)
    user_id = helpers.resolve_id(mgr.resolve_ids, identifier, 'username')

    if remove or refresh:
        mgr.remove_api_authentication_key(user_id)
        click.secho('Successfully removed API authentication key', fg='green')

    if add or refresh:
        api_authentication_key = mgr.add_api_authentication_key(user_id)
        click.secho(f'Successfully added. New API Authentication Key: {api_authentication_key}', fg='green')



"""Check if a server is ready."""
# :license: MIT, see LICENSE for more details.

import click

import SoftLayer
from SoftLayer.CLI import environment
from SoftLayer.CLI import exceptions
from SoftLayer.CLI import helpers


@click.command(cls=SoftLayer.CLI.command.SLCommand, )
@click.argument('identifier')
@click.option('--wait', default=0, show_default=True, type=click.INT,
              help="Seconds to wait")
@environment.pass_env
def cli(env, identifier, wait):
    """Check if a server is ready."""

    compute = SoftLayer.HardwareManager(env.client)
    compute_id = helpers.resolve_id(compute.resolve_ids, identifier,
                                    'hardware')
    ready = compute.wait_for_ready(compute_id, wait)
    if ready:
        env.fout("READY")
    else:
        raise exceptions.CLIAbort("Server %s not ready" % compute_id)



"""https://cloud.ibm.com/docs/CDN?topic=CDN-cdn-deprecation"""
# :license: MIT, see LICENSE for more details.

import click

import SoftLayer


@click.command(cls=SoftLayer.CLI.command.SLCommand, deprecated=True)
def cli():
    """https://cloud.ibm.com/docs/CDN?topic=CDN-cdn-deprecation"""



getAllObjects = [
    {
        "description": "All Datacenters in the USA and Canada.",
        "id": 1,
        "locationGroupTypeId": 1,
        "name": "US/Canada",
                "securityLevelId": None,
                "locationGroupType": {
                    "name": "VDR"
                }
    },
    {
        "description": "All Datacenters in Amsterdam, London, Paris, and Madrid.",
        "id": 2,
        "locationGroupTypeId": 1,
        "name": "AMS/LON/MAD/PAR",
                "securityLevelId": None,
                "locationGroupType": {
                    "name": "VDR"
                }
    },
    {
        "description": "All Datacenters in Singapore, Hong Kong and Japan.",
        "id": 3,
        "locationGroupTypeId": 1,
        "name": "SNG/HKG/JPN",
                "securityLevelId": None,
                "locationGroupType": {
                    "name": "VDR"
                }
    },
    {
        "description": "All Datacenters in Australia",
        "id": 222,
        "locationGroupTypeId": 1,
        "name": "AUS",
                "securityLevelId": None,
                "locationGroupType": {
                    "name": "VDR"
                }
    },
    {
        "description": "All Datacenters in Mexico",
        "id": 262,
        "locationGroupTypeId": 1,
        "name": "MEX",
                "securityLevelId": None,
                "locationGroupType": {
                    "name": "VDR"
                }
    },
    {
        "description": "All Datacenters in Brazil",
        "id": 665,
        "locationGroupTypeId": 1,
        "name": "BRA",
                "securityLevelId": None,
                "locationGroupType": {
                    "name": "VDR"
                }
    },
    {
        "description": "All Datacenters in India",
        "id": 745,
        "locationGroupTypeId": 1,
        "name": "IND",
                "securityLevelId": None,
                "locationGroupType": {
                    "name": "VDR"
                }
    },
    {
        "description": "All Datacenters in Italy",
        "id": 863,
        "locationGroupTypeId": 1,
        "name": "ITA",
                "securityLevelId": None,
                "locationGroupType": {
                    "name": "VDR"
                }
    },
    {
        "description": "All Datacenters in South Korea",
        "id": 1023,
        "locationGroupTypeId": 1,
        "name": "KOR",
                "securityLevelId": None,
                "locationGroupType": {
                    "name": "VDR"
                }
    },
    {
        "description": "All Datacenters in Frankfurt",
        "id": 2023,
        "locationGroupTypeId": 1,
        "name": "FRA",
                "securityLevelId": None,
                "locationGroupType": {
                    "name": "VDR"
                }
    }
]



getGuests = [
    {
        "accountId": 11111,
        "createDate": "2019-09-05T17:03:42-06:00",
        "fullyQualifiedDomainName": "NSX-T Manager",
        "hostname": "NSX-T Manager",
        "id": 22222,
        "maxCpu": 16,
        "maxCpuUnits": "CORE",
        "maxMemory": 49152,
        "startCpus": 16,
        "powerState": {
            "keyName": "RUNNING",
            "name": "Running"
        },
        "status": {
            "keyName": "ACTIVE",
            "name": "Active"
        }
    },
    {
        "accountId": 11111,
        "createDate": "2019-09-23T06:00:53-06:00",
        "hostname": "NSX-T Manager2",
        "id": 33333,
        "maxCpu": 12,
        "maxCpuUnits": "CORE",
        "maxMemory": 49152,
        "startCpus": 12,
        "statusId": 1001,
        "powerState": {
            "keyName": "RUNNING",
            "name": "Running"
        },
        "status": {
            "keyName": "ACTIVE",
            "name": "Active"
        }
    }
]



getObject = {
    'id': 1234,
    'globalIdentifier': 'xxxxc-asd',
    'datacenter': {'id': 12, 'name': 'DALLAS21',
                   'description': 'Dallas 21'},
    'billingItem': {
        'id': 6327,
        'recurringFee': 1.54,
        'nextInvoiceTotalRecurringAmount': 16.08,
        'children': [
            {'description': 'test', 'nextInvoiceTotalRecurringAmount': 1},
        ],
        'orderItem': {
            'order': {
                'userRecord': {
                    'username': 'bob',
                }
            }
        }
    },
    'primaryIpAddress': '4.4.4.4',
    'hostname': 'testtest1',
    'domain': 'test.sftlyr.ws',
    'bareMetalInstanceFlag': True,
    'fullyQualifiedDomainName': 'testtest1.test.sftlyr.ws',
    'processorPhysicalCoreAmount': 4,
    'memoryCapacity': 4,
    'primaryBackendIpAddress': '10.4.4.4',
    'networkManagementIpAddress': '10.4.4.4',
    'hardwareStatus': {'status': 'ACTIVE'},
    'primaryNetworkComponent': {'maxSpeed': 1000, 'speed': 1000},
    'provisionDate': '2020-08-01 15:23:45',
    'notes': 'NOTES NOTES NOTES',
    'operatingSystem': {
        'softwareLicense': {
            'softwareDescription': {
                'referenceCode': 'UBUNTU_20_64',
                'name': 'Ubuntu',
                'version': 'Ubuntu 20.04 LTS',
            }
        },
        'passwords': [
            {'username': 'root', 'password': 'xxxxxxxxxxxx'}
        ],
    },
    'remoteManagementAccounts': [
        {'username': 'root', 'password': 'zzzzzzzzzzzzzz'}
    ],
    'networkVlans': [
        {
            'networkSpace': 'PRIVATE',
            'vlanNumber': 1234,
            'id': 11111
        },
    ],
    'tagReferences': [
        {'tag': {'name': 'a tag'}}
    ],
}

allowAccessToNetworkStorageList = True

getSensorData = [
    {
        "lowerCritical": "5.000",
        "lowerNonCritical": "10.000",
        "sensorId": "Ambient 1 Temperature",
        "sensorReading": "25.000",
        "sensorUnits": "degrees C",
        "status": "ok",
        "upperCritical": "43.000",
        "upperNonCritical": "41.000",
        "upperNonRecoverable": "46.000"
    },
    {
        "lowerCritical": "3500.000",
        "lowerNonCritical": "3700.000",
        "sensorId": "Fan 1 Tach",
        "sensorReading": "6580.000",
        "sensorUnits": "RPM",
        "status": "ok",
        "upperCritical": "25400.000",
        "upperNonCritical": "25300.000",
    }, {
        "sensorId": "IPMI Watchdog",
        "sensorReading": "0x0",
        "sensorUnits": "discrete",
        "status": "0x0080"
    }, {
        "sensorId": "Avg Power",
        "sensorReading": "70.000",
        "sensorUnits": "Watts",
        "status": "ok"
    },
    {
        "lowerCritical": "10.536",
        "lowerNonCritical": "10.780",
        "lowerNonRecoverable": "10.170",
        "sensorId": "12V",
        "sensorReading": "12.305",
        "sensorUnits": "Volts",
        "status": "ok",
        "upperCritical": "13.281",
        "upperNonCritical": "12.915",
        "upperNonRecoverable": "13.403"
    }]

getSoftwareComponents = [{
    "hardwareId": 123456,
    "id": 67064532,
    "passwords": [
        {
            "id": 77995567,
            "notes": "testslcli1",
            "password": "123456",
            "softwareId": 67064532,
            "username": "testslcli1",
        },
        {
            "id": 77944803,
            "notes": "testslcli2",
            "password": "test123",
            "softwareId": 67064532,
            "username": "testslcli2",
        }
    ],
    "softwareLicense": {
        "id": 21854,
        "softwareDescriptionId": 2914,
        "softwareDescription": {
            "id": 2914,
            "longDescription": "Ubuntu 20.04.1-64",
            "name": "Ubuntu",
            "referenceCode": "UBUNTU_20_64",
            "version": "20.04.1-64"
        }
    }
}]



"""Edit SSL certificate."""
# :license: MIT, see LICENSE for more details.

import click

import SoftLayer
from SoftLayer.CLI import environment


@click.command(cls=SoftLayer.CLI.command.SLCommand, )
@click.argument('identifier')
@click.option('--crt',
              type=click.Path(exists=True),
              help="Certificate file")
@click.option('--csr',
              type=click.Path(exists=True),
              help="Certificate Signing Request file")
@click.option('--icc',
              type=click.Path(exists=True),
              help="Intermediate Certificate file")
@click.option('--key', type=click.Path(exists=True), help="Private Key file")
@click.option('--notes', help="Additional notes")
@environment.pass_env
def cli(env, identifier, crt, csr, icc, key, notes):
    """Edit SSL certificate."""
    template = {'id': identifier}
    with open(crt, encoding="utf-8") as file_crt:
        if crt:
            template['certificate'] = file_crt.read()
    with open(key, encoding="utf-8") as file_key:
        if key:
            template['privateKey'] = file_key.read()
    with open(csr, encoding="utf-8") as file_csr:
        if csr:
            template['certificateSigningRequest'] = file_csr.read()
    with open(icc, encoding="utf-8") as file_icc:
        if icc:
            template['intermediateCertificate'] = file_icc.read()
    if notes:
        template['notes'] = notes

    manager = SoftLayer.SSLManager(env.client)
    manager.edit_certificate(template)



getDatacenters = [
    {
        "id": 1441195,
        "longName": "Dallas 10",
        "name": "dal10"
    },
    {
        "id": 449494,
        "longName": "Dallas 9",
        "name": "dal09"
    }
]

getHardwareRouters = []



"""List active quotes on an account."""
# :license: MIT, see LICENSE for more details.
import click

from SoftLayer.CLI.command import SLCommand as SLCommand
from SoftLayer.CLI import environment
from SoftLayer.CLI import formatting
from SoftLayer.managers import ordering
from SoftLayer.utils import clean_time


@click.command(cls=SLCommand)
@environment.pass_env
def cli(env):
    """List all active quotes on an account"""
    table = formatting.Table([
        'Id', 'Name', 'Created', 'Expiration', 'Status', 'Package Name', 'Package Id'
    ])
    table.align['Name'] = 'l'
    table.align['Package Name'] = 'r'
    table.align['Package Id'] = 'l'

    manager = ordering.OrderingManager(env.client)
    items = manager.get_quotes()

    for item in items:
        package = item['order']['items'][0]['package']
        table.add_row([
            item.get('id'),
            item.get('name'),
            clean_time(item.get('createDate')),
            clean_time(item.get('modifyDate')),
            item.get('status'),
            package.get('keyName'),
            package.get('id')
        ])
    env.fout(table)



"""Cancel an existing iSCSI account."""
# :license: MIT, see LICENSE for more details.

import click

import SoftLayer
from SoftLayer.CLI import environment
from SoftLayer.CLI import exceptions
from SoftLayer.CLI import formatting


@click.command(cls=SoftLayer.CLI.command.SLCommand, )
@click.argument('volume-id')
@click.option('--reason', help="An optional reason for cancellation")
@click.option('--immediate',
              is_flag=True,
              help="Cancels the block storage volume immediately instead "
                   "of on the billing anniversary")
@click.option('--force', default=False, is_flag=True, help="Force cancel block volume without confirmation")
@environment.pass_env
def cli(env, volume_id, reason, immediate, force):
    """Cancel an existing block storage volume.

    Example::
        slcli block volume-cancel 12345678 --immediate -f
        This command cancels volume with ID 12345678 immediately and without asking for confirmation.
"""

    block_storage_manager = SoftLayer.BlockStorageManager(env.client)

    if not force:
        if not (env.skip_confirmations or
                formatting.confirm(f"This will cancel the block volume: {volume_id} and cannot be undone. Continue?")):
            raise exceptions.CLIAbort('Aborted')

    cancelled = block_storage_manager.cancel_block_volume(volume_id,
                                                          reason, immediate)

    if cancelled:
        if immediate:
            click.echo('Block volume with id %s has been marked'
                       ' for immediate cancellation' % volume_id)
        else:
            click.echo('Block volume with id %s has been marked'
                       ' for cancellation' % volume_id)
    else:
        click.echo('Unable to cancel block volume %s' % volume_id)



"""Edit a subnet."""
# :license: MIT, see LICENSE for more details.

import click

import SoftLayer
from SoftLayer.CLI import environment
from SoftLayer.CLI import helpers


@click.command(cls=SoftLayer.CLI.command.SLCommand, short_help="Edit note and tags of a subnet")
@click.argument('identifier')
@click.option('--tags', '-t', type=click.STRING,
              help='Comma separated list of tags, enclosed in quotes. "tag1, tag2"')
@click.option('--note', '-n', type=click.STRING,
              help="The note")
@environment.pass_env
def cli(env, identifier, tags, note):
    """Edit note and tags of a subnet."""

    mgr = SoftLayer.NetworkManager(env.client)
    subnet_id = helpers.resolve_id(mgr.resolve_subnet_ids, identifier,
                                   name='subnet')

    if tags:
        result = mgr.set_tags_subnet(subnet_id, tags)
        print_result(result, "Set tags")

    if note:
        result = mgr.edit_note_subnet(subnet_id, note)
        print_result(result, "Edit note")


def print_result(result, detail):
    """Prints a successfully or Failed message."""
    if result:
        click.secho(f"{detail} successfully", fg='green')
    else:
        click.secho(f"Failed to {detail.lower()}", fg='red')
